import os
import sys
import json
import unittest
import configparser
from phanterpwa.configer import ProjectConfig
from phanterpwa.tools import interpolate

CURRENT_DIR = os.path.normpath(os.path.join(os.path.dirname(__file__)))
ENV_PYTHON = os.path.normpath(sys.executable)

sample_cfg = """{
  "CONFIG_INDENTIFY": "project_config",
  "ENVIRONMENT": {
    "python": "{{pyenv}}",
    "path": "{{pypath}}"
  },
  "PROJECT": {
    "name": "project01",
    "title": "PhanterPWA",
    "version": "0.0.1",
    "compilation": 0,
    "author": "PhanterJR<phanterjr@conexaodidata.com.br>",
    "debug": true,
    "packaged": true,
    "minify": false,
    "baseport": 10000,
    "basehost": "127.0.0.1",
    "path": "{{projectpath}}"
  },
  "FRONTEND": {
    "app01": {
      "build_folder": "{{app01}}",
      "title": "app01",
      "transcrypt_main_file": "application",
      "styles_main_file": "application",
      "views_main_file": "index",
      "timeout_to_resign": 600,
      "host": "127.0.0.1",
      "port": 10001
    },
    "app02": {
      "build_folder": "{{app02}}",
      "title": "app02",
      "transcrypt_main_file": "application",
      "styles_main_file": "application",
      "views_main_file": "index",
      "timeout_to_resign": 600,
      "host": "127.0.0.1",
      "port": 10002
    }
  },
  "BACKEND": {
    "api": {
      "title": "PhanterPWA Backend",
      "default_time_user_token_expire": 7200,
      "default_time_user_token_expire_remember_me": 2592000,
      "default_time_csrf_token_expire": 4200,
      "default_time_temporary_password_expire": 4200,
      "timeout_to_resend_temporary_password_mail": 3900,
      "default_time_client_token_expire": 63072000,
      "default_time_activation_code_expire": 3600,
      "default_time_two_factor_code_expire": 3600,
      "wait_time_to_try_activate_again": 3900,
      "timeout_to_resend_activation_email": 300,
      "timeout_to_resign": 600,
      "timeout_to_next_login_attempt": 4200,
      "max_login_attempts": 5,
      "max_activation_attempts": 5,
      "host": "127.0.0.1",
      "port": 10000,
      "secret_key": "{{secret_key}}",
      "url_secret_key": "{{url_secret_key}}"
    }
  }
}"""

sample_cfg2 = """{
    "CONFIG_INDENTIFY": "project_config",
    "ENVIRONMENT": {
        "python": "{{pyenv}}",
        "path": "{{pypath}}"
    },
    "PROJECT": {
        "name": "project_without_confgjson_and_with_ini",
        "title": "CHANGED",
        "version": "0.0.1",
        "compilation": 0,
        "author": "PhanterJR<phanterjr@conexaodidata.com.br>",
        "debug": true,
        "packaged": true,
        "path": "{{projectpath}}"
    },
    "API": {
        "secret_key": "{{secret_key}}",
        "url_secret_key": "{{url_secret_key}}",
        "default_time_user_token_expire": 7200,
        "default_time_user_token_expire_remember_me": 2592000,
        "default_time_csrf_token_expire": 4200,
        "default_time_temporary_password_expire": 4200,
        "timeout_to_resend_temporary_password_mail": 3900,
        "default_time_client_token_expire": 63072000,
        "default_time_activation_code_expire": 3600,
        "default_time_two_factor_code_expire": 3600,
        "wait_time_to_try_activate_again": 3900,
        "timeout_to_resend_activation_email": 300,
        "timeout_to_resign": 600,
        "timeout_to_next_login_attempt": 4200,
        "max_login_attempts": 5,
        "max_activation_attempts": 5,
        "remote_address_debug": "http://127.0.0.1:8881",
        "websocket_address_debug": "ws://127.0.0.1:8881/api/websocket",
        "remote_address": "https://your_domain.com",
        "websocket_address": "wss://your_domain.com/api/websocket",
        "host": "127.0.0.1",
        "port": 8881
    },
    "FRONTEND": {
        "app01": {
            "build_folder": "{{app01}}",
            "title": "app01",
            "timeout_to_resign": 600,
            "transcrypt_main_file": "application",
            "styles_main_file": "application",
            "views_main_file": "index",
            "host": "127.0.0.1",
            "port": 8882
        },
        "app02": {
            "build_folder": "{{app02}}",
            "title": "app02",
            "timeout_to_resign": 600,
            "transcrypt_main_file": "application",
            "styles_main_file": "application",
            "views_main_file": "index",
            "host": "127.0.0.1",
            "port": 8883
        }
    },
    "EMAIL": {
        "server": "mail.yourservermail.com",
        "username": "username@yourservermail.com",
        "default_sender": "contato@conexaodidata.com.br",
        "password": "{{password}}",
        "port": 465,
        "use_tls": false,
        "use_ssl": true
    },
    "CONTENT_EMAILS": {
        "copyright": "Conex\u00e3o Didata \u00a9 2011-{{now}}",
        "link_to_your_site": "https://phanterpwa.conexaodidata.com.br"
    }
}"""

api_sample = """[API]
default_time_user_token_expire = 7200
default_time_user_token_expire_remember_me = 2592000
default_time_csrf_token_expire = 4200
default_time_temporary_password_expire = 4200
timeout_to_resend_temporary_password_mail = 3900
default_time_client_token_expire = 63072000
default_time_activation_code_expire = 3600
default_time_two_factor_code_expire = 3600
wait_time_to_try_activate_again = 3900
timeout_to_resend_activation_email = 300
timeout_to_resign = 600
timeout_to_next_login_attempt = 4200
max_login_attempts = 5
max_activation_attempts = 5
remote_address_debug = http://127.0.0.1:8881
websocket_address_debug = ws://127.0.0.1:8881/api/websocket
remote_address = https://your_domain.com
websocket_address = wss://your_domain.com/api/websocket
host = 127.0.0.1
port = 8881

"""
project_sample = """[PROJECT]
title = CHANGED
version = 0.0.1
author = PhanterJR<phanterjr@conexaodidata.com.br>
debug = True
packaged = True

[EMAIL]
server = mail.yourservermail.com
username = username@yourservermail.com
default_sender = contato@conexaodidata.com.br
port = 465
use_tls = False
use_ssl = True

[CONTENT_EMAILS]
copyright = Conexão Didata © 2011-{{now}}
link_to_your_site = https://phanterpwa.conexaodidata.com.br

"""
project_sample2 = """[PROJECT]
title = PhanterPWA
version = 0.0.1
author = PhanterJR<phanterjr@conexaodidata.com.br>
debug = True
packaged = True

[EMAIL]
server = mail.yourservermail.com
username = username@yourservermail.com
default_sender = contato@conexaodidata.com.br
port = 465
use_tls = False
use_ssl = True

[CONTENT_EMAILS]
copyright = Conexão Didata © 2011-{{now}}
link_to_your_site = https://phanterpwa.conexaodidata.com.br

"""


class TestProjectConfig(unittest.TestCase):
    def test0_documentation_example(self):
        # module doc
        import phanterpwa
        path_dir = os.path.join(os.path.dirname(phanterpwa.__file__), "tests", "test_configer_path", "project01")
        if os.path.isfile(os.path.join(path_dir, "config.json")):
            os.remove(os.path.join(path_dir, "config.json"))
        # if os.path.isfile(os.path.join(path_dir, "backapps", "api", "app.ini")):
        #     os.remove(os.path.join(path_dir, "backapps", "api", "app.ini"))
        if os.path.isfile(os.path.join(path_dir, "project.ini")):
            os.remove(os.path.join(path_dir, "project.ini"))
        cfg = ProjectConfig(path_dir)
        print(cfg._ini_apps_backend)

        self.assertTrue(os.path.isfile(os.path.join(path_dir, "config.json")))
        self.assertTrue(os.path.isfile(os.path.join(path_dir, "backapps", "api", "app.ini")))
        self.assertTrue(os.path.isfile(os.path.join(path_dir, "project.ini")))
        # file
        self.assertEqual(
            cfg.file,
            os.path.join(path_dir, "config.json")
        )
        # config
        string_cfg = json.dumps(cfg.config, ensure_ascii=False, indent=2)
        print(string_cfg)
        s_cfg = interpolate(sample_cfg, {
            "pyenv": ENV_PYTHON.replace("\\", "\\\\"),
            "pypath": os.path.dirname(ENV_PYTHON).replace("\\", "\\\\"),
            "projectpath": path_dir.replace("\\", "\\\\"),
            "app01": os.path.join(path_dir, "frontapps", "app01", "www").replace("\\", "\\\\"),
            "app02": os.path.join(path_dir, "frontapps", "app02", "www").replace("\\", "\\\\")
        })
        self.assertEqual(
            string_cfg,
            s_cfg
        )
        # inis
        self.assertTrue(isinstance(cfg.backend_ini["api"], configparser.ConfigParser))
        self.assertTrue(isinstance(cfg.project_ini, configparser.ConfigParser))
        self.assertTrue(isinstance(cfg.project_secret_ini, configparser.ConfigParser))
        self.assertTrue(isinstance(cfg.frontend_ini["app01"], configparser.ConfigParser))
        self.assertTrue(isinstance(cfg.frontend_ini["app02"], configparser.ConfigParser))
        # save
        self.assertEqual(
            cfg['PROJECT']['title'],
            "PhanterPWA"
        )
        cfg['PROJECT']['title'] = 'PhanterPWA2'
        cfg.save()
        with open(os.path.join(path_dir, "config.json"), encoding="utf-8") as f:
            content = json.load(f)
        content
        self.assertEqual(
            content['PROJECT']['title'],
            "PhanterPWA2"
        )

    # def test1(self):
    #     path_dir = os.path.join(CURRENT_DIR, 'test_configer_path', 'project_without_ini')
    #     if os.path.isfile(os.path.join(path_dir, "project.ini")):
    #         os.remove(os.path.join(path_dir, "project.ini"))
    #     if os.path.isfile(os.path.join(path_dir, "api", "api.ini")):
    #         os.remove(os.path.join(path_dir, "api", "api.ini"))
    #     ProjectConfig(path_dir)
    #     self.assertTrue(os.path.isfile(os.path.join(path_dir, "config.json")))
    #     self.assertTrue(os.path.isfile(os.path.join(path_dir, "api", "api.ini")))
    #     self.assertTrue(os.path.isfile(os.path.join(path_dir, "project.ini")))
    #     with open(os.path.join(path_dir, "api", "api.ini"), "r", encoding="utf-8") as f:
    #         apicontent = f.read()
    #     self.assertEqual(apicontent, api_sample)
    #     with open(os.path.join(path_dir, "project.ini"), "r", encoding="utf-8") as f:
    #         projectcontent = f.read()
    #     self.assertEqual(projectcontent, project_sample2)

    # def test2(self):
    #     path_dir = os.path.join(CURRENT_DIR, 'test_configer_path', 'project_without_confgjson_and_with_ini')
    #     if os.path.isfile(os.path.join(path_dir, "config.json")):
    #         os.remove(os.path.join(path_dir, "config.json"))
    #     with open(os.path.join(path_dir, "project.ini"), 'w', encoding='utf-8') as f:
    #         f.write(project_sample2)

    #     cfg = ProjectConfig(path_dir)
    #     self.assertTrue(os.path.isfile(os.path.join(path_dir, "config.json")))
    #     self.assertTrue(os.path.isfile(os.path.join(path_dir, "api", "api.ini")))
    #     self.assertTrue(os.path.isfile(os.path.join(path_dir, "project.ini")))
    #     self.assertEqual(cfg['PROJECT']['title'], 'PhanterPWA')
    #     self.assertEqual(cfg.project_ini['PROJECT']['title'], 'PhanterPWA')

    #     cfg['PROJECT']['title'] = "CHANGED"
    #     cfg.save()
    #     with open(os.path.join(path_dir, "config.json"), "r", encoding="utf-8") as f:
    #         cfgcontent = json.load(f)
    #         cfgcontent = json.dumps(cfgcontent, ensure_ascii=False, indent=4)
    #     s_cfg = interpolate(sample_cfg2, {
    #         "pyenv": ENV_PYTHON.replace("\\", "\\\\"),
    #         "pypath": os.path.dirname(ENV_PYTHON).replace("\\", "\\\\"),
    #         "projectpath": path_dir.replace("\\", "\\\\"),
    #         "app01": os.path.join(path_dir, "frontapps", "app01", "www").replace("\\", "\\\\"),
    #         "app02": os.path.join(path_dir, "frontapps", "app02", "www").replace("\\", "\\\\")
    #     })
    #     self.assertEqual(cfgcontent, s_cfg)
    #     with open(os.path.join(path_dir, "api", "api.ini"), "r", encoding="utf-8") as f:
    #         apicontent = f.read()
    #     self.assertEqual(apicontent, api_sample)
    #     with open(os.path.join(path_dir, "project.ini"), "r", encoding="utf-8") as f:
    #         projectcontent = f.read()
    #     self.assertEqual(projectcontent, project_sample)


if __name__ == '__main__':
    unittest.main()



from org.transcrypt.stubs.browser import __pragma__
import phanterpwa.frontend.helpers as helpers

__pragma__('alias', "jQuery", "$")
__pragma__('skip')

# it is ignored on transcrypt
window = jQuery = console = document = localStorage = URL = WebSocket = FormData = setTimeout = WebSocket =\
    sessionStorage = this = confirm = JSON = js_undefined = navigator = __new__ = Date = location = 0

__pragma__('noskip')


DIV = helpers.XmlConstructor.tagger("div")
H2 = helpers.XmlConstructor.tagger("h2")
XML = helpers.XML
TEXTAREA = helpers.XmlConstructor.tagger("textarea")


__pragma__('kwargs')


class WebSocketPhanterPWA():
    def __init__(self, websocket_address, **parameters):
        self._opened = False
        self.websocket_address = websocket_address
        self.comulative_time = 300
        self.last_ws = None
        self.manual_connection = False
        self.manual_close = False
        self.onClose = None
        self.onError = None
        self.onMessage = None
        self.onOpen = None
        setTimeout(lambda: self.start(), self.comulative_time)

    def send(self, message):
        if self._opened:
            self._authorization = window.PhanterPWA.get_authorization()
            if self._authorization is not None:
                __pragma__("jsiter")
                msg = JSON.stringify({
                    "phanterpwa-authorization": self._authorization,
                    "message": message
                })
                __pragma__("nojsiter")
                self._ws.send(str('{0}'.format(msg)))
                if window.PhanterPWA.DEBUG:
                    if message is "command_online":
                        console.info("__ Server, I'm online?")
                    elif message is "command_offline":
                        console.info("__ Server, I'm stay offline. Bye.")
            else:
                if message not in ["command_online", "command_offline"]:
                    __pragma__("jsiter")
                    msg = JSON.stringify(message)
                    __pragma__("nojsiter")
                    self._ws.send(msg)

    def close(self):
        self.manual_close = True
        self._ws.close()

    def _onClose(self, evt):
        self._opened = False
        if window.PhanterPWA.DEBUG:
            console.info("Closing websocket")
        if not self.manual_close:
            setTimeout(lambda: self.start(), self.comulative_time)
        else:
            self.manual_close = False
        if callable(self.onClose):
            self.onClose(evt)

    def _onError(self, evt):
        if window.PhanterPWA.DEBUG:
            console.info("Error on websocket", evt)
        if self.comulative_time > 9000 and not self.manual_connection:
            window.PhanterPWA.flash("Lost server connection!")
            self.manual_connection = True
        else:
            self.comulative_time = self.comulative_time + 1000
        if callable(self.onError):
            self.onError(evt)

    def _onMessage(self, evt):
        if window.PhanterPWA.DEBUG:
            console.info(evt.data)
        if callable(self.onMessage):
            self.onMessage(evt)

    def _onOpen(self, evt):
        self.comulative_time = 0
        self._opened = True
        if window.PhanterPWA.DEBUG:
            console.info("Opening websocket")
        self.send("command_online")
        if callable(self.onOpen):
            self.onOpen(evt)

    def _binds(self):
        self._ws.onopen = lambda evt: self._onOpen(evt)
        self._ws.onmessage = lambda evt: self._onMessage(evt)
        self._ws.onerror = lambda evt: self._onError(evt)
        self._ws.onclose = lambda evt: self._onClose(evt)

    def start(self):
        if not self.manual_connection:
            self._ws = __new__(WebSocket(self.websocket_address))
            self._authorization = window.PhanterPWA.get_authorization()
            self._config = window.PhanterPWA.CONFIG
            self._binds()
        else:
            r = confirm("It's try connect again?")
            if(r):
                self.manual_connection = False
                self.comulative_time = 0
                self._ws = __new__(WebSocket(self.websocket_address))
                self._authorization = window.PhanterPWA.get_authorization()
                self._config = window.PhanterPWA.CONFIG
                self._binds()


__pragma__('nokwargs')



from core import projectConfig as CONFIG
from phanterpwa.helpers import (
    DIV,
    A,
    I,
    HTML,
    HEAD,
    BODY,
    META,
    TITLE,
    NAV,
    UL,
    LI,
    MAIN,
    FOOTER,
    SPAN,
    XML,
    H4,
    LINK
)
from .extends.svg_logo import html as SVG_LOGO
from .extends.javascript_head import html as JAVASCRIPT_HEAD
from .extends.css_head import html as CSS_HEAD
from .extends.favicons import html as FAVICONS
from .extends.captcha_preload import PRELOADER

app_version = CONFIG['PROJECT']['version']
app_name = CONFIG['PROJECT']['name']

html = HTML(
    HEAD(
        TITLE(app_name),
        META(_charset="utf-8"),
        META(
            _name="viewport",
            _content="width=device-width, initial-scale=1.0, maximum-scale=1.0, user-scalable=no"
        ),
        META(_name="aplication-name", _content="Flask, Nginx, Cordova"),
        META(_name="aplication-version", _content=app_version),
        META(_name="msapplication-tap-highlight", _content="no"),
        # LINK(_rel="manifest", _href="/static/{0}/json/manifest.json".format(app_version)),
        CSS_HEAD,
        JAVASCRIPT_HEAD,
        FAVICONS
    ),
    BODY(
        DIV(_id="layout-top_slide-container"),
        NAV(
            DIV(
                DIV(
                    _id="layout-main_button-container",
                    _class="e-float_left"
                ),
                DIV(
                    DIV(
                        XML(SVG_LOGO),
                        _class="logo-svg"
                    ),
                    _class="p-logo link"
                ),
                _class="nav-wrapper"
            ),
            _class="main-nav"
        ),
        DIV(
            DIV(
                _id="layout-left_bar-container"
            ),
            MAIN(
                DIV(
                    PRELOADER,
                    _style="width:100%; text-align: center; padding-top: 100px;"
                ),
                _id="main-container"
            ),
            _class="main-and-left_bar"
        ),
        DIV(
            _id="modal-container"),
        FOOTER(
            DIV(
                _id="main-progress-bar-container",
                _class="main-progress-bar-container"
            ),
            DIV(
                DIV(_class="row"),
                _class='container'
            ),
            DIV(
                DIV(
                    A(
                        "Conexão Didata © 2011-", SPAN("2020", _id="conexao_year"),
                        _class="copyright",
                        _href="https://www.conexaodidata.com.br",
                        _target="blank"
                    ),
                    A(
                        "PhanterJR",
                        _class="e-float_right",
                        _href="https://phanterjr.conexaodidata.com.br",
                        _target="blank"
                    ),
                    _class="phanterpwa-container"
                ),
                _class="footer-copyright grey darken-3"
            ),
            _class="page-footer main-footer grey darken-4"
        ),
    ),
    _lang="pt-BR"
)



import unittest
from phanterpwa.tests import test_cli
from phanterpwa.tests import test_components_preloaders
from phanterpwa.tests import test_helpers
from phanterpwa.tests import test_i18n
from phanterpwa.tests import test_reversexml
from phanterpwa.tests import test_tools
from phanterpwa.tests import test_configer

test_list = [
    "test_cli",
    "test_components_preloaders",
    "test_helpers",
    "test_i18n",
    "test_reversexml",
    "test_tools",
    "test_configer",
]


def run():
    suite = unittest.TestLoader().loadTestsFromModule(test_cli)
    unittest.TextTestRunner(verbosity=2).run(suite)
    suite = unittest.TestLoader().loadTestsFromModule(test_components_preloaders)
    unittest.TextTestRunner(verbosity=2).run(suite)
    suite = unittest.TestLoader().loadTestsFromModule(test_helpers)
    unittest.TextTestRunner(verbosity=2).run(suite)
    suite = unittest.TestLoader().loadTestsFromModule(test_i18n)
    unittest.TextTestRunner(verbosity=2).run(suite)
    suite = unittest.TestLoader().loadTestsFromModule(test_reversexml)
    unittest.TextTestRunner(verbosity=2).run(suite)
    suite = unittest.TestLoader().loadTestsFromModule(test_tools)
    unittest.TextTestRunner(verbosity=2).run(suite)
    suite = unittest.TestLoader().loadTestsFromModule(test_configer)
    unittest.TextTestRunner(verbosity=2).run(suite)


if __name__ == '__main__':
    run()



import phanterpwa.frontend.gatehandler as gatehandler
import phanterpwa.frontend.helpers as helpers
import phanterpwa.frontend.components.widgets as widgets
import phanterpwa.frontend.components.left_bar as left_bar
from org.transcrypt.stubs.browser import __pragma__

__pragma__('alias', "jQuery", "$")
__pragma__('skip')
jQuery = sessionStorage = JSON = M = js_undefined = window = setTimeout = \
    __new__ = FormData = console = localStorage = document = this = 0
__pragma__('noskip')

DIV = helpers.XmlConstructor.tagger("div")
I = helpers.XmlConstructor.tagger("i")
H1 = helpers.XmlConstructor.tagger("h1")
H2 = helpers.XmlConstructor.tagger("h2")
H3 = helpers.XmlConstructor.tagger("h3")
STRONG = helpers.XmlConstructor.tagger("strong")
SPAN = helpers.XmlConstructor.tagger("span")
A = helpers.XmlConstructor.tagger("a")
UL = helpers.XmlConstructor.tagger("ul")
LI = helpers.XmlConstructor.tagger("li")
LABEL = helpers.XmlConstructor.tagger("label")
XSECTION = helpers.XSECTION
I18N = helpers.I18N
CONCATENATE = helpers.CONCATENATE
XTABLE = widgets.Table
XML = helpers.XML
XTRD = widgets.TableData
XTRH = widgets.TableHead
XFOOTER = widgets.TableFooterPagination


__pragma__('kwargs')


class Index(gatehandler.Handler):
    def __init__(self, request):
        gatehandler.Handler.__init__(self, request)
        self.projects_data = localStorage.getItem("admin_projects_data")

    def initialize(self):
        html = CONCATENATE(
            DIV(
                DIV(
                    DIV(
                        DIV("DEVELOPMENT", _class="phanterpwa-breadcrumb"),
                        _class="phanterpwa-breadcrumb-wrapper"
                    ),
                    _class="p-container"),
                _class='title_page_container card'
            ),
            DIV(
                DIV(
                    DIV(
                        DIV(
                            widgets.Preloaders("preload_android"),
                            _style="width:100%; text-align: center; padding-top: 100px; padding-bottom: 100px;"
                        ),
                        _id="projects_container"),
                    _class="card p-row e-padding_10"
                ),
                _class="phanterpwa-container p-container"
            )
        )
        html.html_to("#main-container")
        BackButton = left_bar.LeftBarButton(
            "back_localizar_aluno",
            "Voltar",
            I(_class="fas fa-arrow-circle-left"),
            **{"_phanterpwa-way": "home",
                "position": "top",
                "ways": ["developer"]}
        )
        TestButton = left_bar.LeftBarButton(
            "test_phanterpwa",
            "Test PhanterPWA",
            I(_class="fas fa-tasks"),
            **{"_phanterpwa-way": "test_phanterpwa",
                "position": "top",
                "ways": ["developer"]}
        )
        CheckRequeriments = left_bar.LeftBarButton(
            "check_requeriments",
            "Check Requeriments",
            I(_class="fab fa-connectdevelop"),
            **{"_phanterpwa-way": "check_requeriments",
                "position": "top",
                "ways": ["developer"]}
        )
        NewProjectsFolder = left_bar.LeftBarButton(
            "change_projects_folder",
            "Change Project Folder",
            I(_class="fas fa-folder-open"),
            **{
                "position": "top",
                "ways": ["developer"],
            }
        )
        window.PhanterPWA.Components['left_bar'].add_button(BackButton)
        window.PhanterPWA.Components['left_bar'].add_button(TestButton)
        window.PhanterPWA.Components['left_bar'].add_button(CheckRequeriments)
        window.PhanterPWA.Components['left_bar'].add_button(NewProjectsFolder)
        if self.projects_data is None or self.projects_data is js_undefined:
            window.PhanterPWA.GET(**{
                'url_args': ['api', 'projects'],
                'onComplete': self._after_get_applications
            })
        else:
            self._xml_projects_list(self.projects_data)
        self._binds()

    def _binds(self):
        jQuery("#phanterpwa-component-left_bar-menu_button-change_projects_folder").off(
            "click.change_projects_folder"
        ).on(
            "click.change_projects_folder",
            self.change_projects_folder,
        )

    def _bind_btn_run_stop(self, el):
        jQuery(el).find(".btn_run_stop_project").off(
            "click.run_stop_project"
        ).on(
            "click.run_stop_project",
            lambda: self._run_stop_project(this)
        )

    def _run_stop_project(self, el):
        el = jQuery(el)
        project_path = el.attr("data-path")
        cmd = el.text().lower()
        admin_authorization = localStorage.getItem("admin_authorization")
        window.PhanterPWA.WS.onMessage = lambda ev: self._after_get_projects_folder(ev)
        window.PhanterPWA.WS.send({
            "authorization": admin_authorization,
            "command": cmd,
            "project_path": project_path
        })

    def reload(self):
        self._binds()

    def change_projects_folder(self):
        admin_authorization = localStorage.getItem("admin_authorization")
        window.PhanterPWA.WS.onMessage = lambda ev: self._after_get_projects_folder(ev)
        window.PhanterPWA.WS.send({
            "authorization": admin_authorization, "command": "change_project_folder"
        })

    def _after_get_projects_folder(self, evt):
        str_json = JSON.parse(evt.data)
        console.log(str_json)
        if str_json.command == "change_project_folder":
            if str_json.status == 200:
                window.PhanterPWA.open_way("developer")
        elif str_json.command == "check_all":
            if str_json.status == 200:
                jQuery(".phanterpwa-widget-table-data").find(
                    "span.project_status_content").text("stopped").removeClass("stopped")
                for r in str_json.project_running:
                    jQuery(".phanterpwa-widget-table-data").find(
                        "span.project_status_content['data-path'={0}]".format(r)).text("running").addClass("running")
        elif str_json.command == "run" or str_json.command == "stop":
            if str_json.status == 200:
                window.PhanterPWA.GET(**{
                    'url_args': ['api', 'projects'],
                    'onComplete': self._after_get_applications
                })

    def _run_project(self, project_path):
        admin_authorization = localStorage.getItem("admin_authorization")
        window.PhanterPWA.WS.onMessage = lambda ev: self._after_get_projects_folder(ev)
        window.PhanterPWA.WS.send({
            "authorization": self.admin_authorization,
            "command": "run",
            "project_path": project_path
        })

    def _xml_projects_list(self, json):
        table = XTABLE(
            "projects-table",
            XTRH(
                "projects-table-head",
                *["Project Name", "Diretory", "Status", "Port"],
                DIV(
                    I(_class="fas fa-plus"),
                    **{
                        "_phanterpwa-way": "project/novo",
                        "_class": "icon_button wave_on_click"
                    }
                )
            )
        )
        for x in json.projects_list:
            btn_lbl = "run"
            if x[2] == "running":
                btn_lbl = "stop"
            table.append(
                XTRD(
                    "projects-table-data-{0}".format(x[0]),
                    x[0],
                    x[1],
                    SPAN(
                        x[2].capitalize(),
                        **{
                            "_class": "project_status_content {0}".format(x[2]),
                            "_id": "project_status_{0}".format(x[0]),
                            "_data-path": x[1],
                        }

                    ),
                    x[3],
                    widgets.MenuBox(
                        "drop_project_{0}".format(x[0]),
                        DIV(I(_class="fas fa-ellipsis-v"), _class="icon_button wave_on_click"),
                        custom_menu=UL(
                            LI(btn_lbl.capitalize(), **{
                                "_id": "btn_run_stop_project_{0}".format(x[0]),
                                "_class": "btn_run_stop_project",
                                "_data-path": x[1]
                            }),
                            LI("View", **{
                                "_class": "botao_editar_role",
                                "_phanterpwa-way": "project/{0}/view".format(x[0])
                            }),
                            LI("Delete", **{
                                "_class": "botao_editar_role",
                                "_phanterpwa-way": "project/{0}/delete".format(x[0])
                            }),
                            **{"data-menubox": "drop_project_{0}".format(x[0]),
                            "_class": 'dropdown-content'},
                        ),
                        onOpen=self._bind_btn_run_stop
                    )
                )
            )

        html = DIV(
            XSECTION(
                LABEL("Enviroment"),
                DIV(
                    DIV(
                        DIV(
                            DIV(
                                STRONG("PYTHON EXECUTABLE"),
                                SPAN(json.enviroment.python_executable),
                                _class="e-tagger-wrapper"
                            ),
                            _class="p-col w1p100"
                        ),
                        _class="p-row"
                    ),
                    DIV(
                        DIV(
                            DIV(
                                STRONG("PYTHON PATH"),
                                SPAN(json.enviroment.python_path),
                                _class="e-tagger-wrapper"
                            ),
                            _class="p-col w1p100"
                        ),
                        _class="p-row"
                    ),
                    DIV(
                        DIV(
                            DIV(
                                STRONG("PYTHON VERSION"),
                                SPAN(json.enviroment.python_version),
                                _class="e-tagger-wrapper"
                            ),
                            _class="p-col w1p100 w3p50"
                        ),
                        DIV(
                            DIV(
                                STRONG("PHANTERPWA VERSION"),
                                SPAN(json.enviroment.phanterpwa_version),
                                _class="e-tagger-wrapper"
                            ),
                            _class="p-col w1p100 w3p50"
                        ),
                        _class="p-row"
                    ),
                    DIV(
                        DIV(
                            DIV(
                                STRONG("PROJECTS FOLDER"),
                                SPAN(json.enviroment.projects_folder),
                                _class="e-tagger-wrapper"
                            ),
                            _class="p-col w1p100"
                        ),
                        _class="p-row"
                    ),
                    _class="e-padding_20"
                )
            ),
            table
        )
        html.html_to("#projects_container")
        self._binds()

    def _after_get_applications(self, data, ajax_status):
        if ajax_status == "success":
            json = data.responseJSON
            if data.status == 200:
                localStorage.setItem("admin_projects_data", JSON.stringify(json))
                self.admin_projects_data = json
                window.PhanterPWA.flash(**{'html': json.i18n.message})

                self._xml_projects_list(json)

            elif data.status == 202:
                window.PhanterPWA.flash(**{'html': json.i18n.message})
            localStorage.setItem("admin_authorization", json.authorization)
            self.admin_authorization = json.authorization
            self._binds()
        else:
            window.PhanterPWA.flash("Problem on server: {0}".format(str(data.status)))


__pragma__('nokwargs')




from core import projectConfig as CONFIG
from phanterpwa.components.preloaders import android

PRELOADER = android.PRELOADER
if CONFIG['PROJECT']["debug"]:
    import os
    from phanterpwa.tools import app_name_from_relative_child
    app_folder_name = app_name_from_relative_child(CONFIG['PROJECT']['path'], __file__)
    PRELOADER._target_sass = os.path.join(
        CONFIG["PROJECT"]["path"], "frontapps", app_folder_name, "sources", "styles", "extends", "capcha_preload.sass")
PRELOADER.sass_vars = {
    'STROKEWIDTH': '10px',
    'CONTAINERWIDTH': '200px',
    'COLOR1': 'blue',
    'COLOR2': 'red',
    'COLOR3': '#f4b400',
    'COLOR4': 'green'
}
html = PRELOADER



from phanterpwa.frontend import helpers
from org.transcrypt.stubs.browser import __pragma__

__pragma__('alias', "jQuery", "$")
__pragma__('skip')

# it is ignored on transcrypt
window = jQuery = console = document = localStorage = Hammer =\
    sessionStorage = this = FileReader = JSON = js_undefined = navigator = __new__ = Date = 0

__pragma__('noskip')


INPUT = helpers.XmlConstructor.tagger("input", True)
I = helpers.XmlConstructor.tagger("i")
DIV = helpers.XmlConstructor.tagger("div")
SELECT = helpers.XmlConstructor.tagger("select")
OPTION = helpers.XmlConstructor.tagger("option")
FORM = helpers.XmlConstructor.tagger("form")
XML = helpers.XML


class phanterpwaSelect():
    def __init__(self, el, parameters=None):
        self.element = jQuery(el)
        self.json_select = None
        self.placeholder = self.element.attr("placeholder")
        self.onButtonClick = None
        if parameters is not None:
            self.element = el
            self.json_select = parameters['json_select']

            if 'placeholder' in parameters:
                self.placeholder = parameters['placeholder']

            if 'onButtonClick' in parameters:
                self.onButtonClick = parameters['onButtonClick']

            self.input_select = self.element.attr('phanterpwa-input-select')
        if self.json_select is None or self.json_select is js_undefined:
            if window.phanterpwaJsonSelects is not None and window.phanterpwaJsonSelects is not js_undefined:
                self.json_select = window.phanterpwaJsonSelects[self.input_select]
        if self.placeholder is None or self.placeholder is js_undefined:
            self.placeholder = ""

    def start(self):
        if self.input_select is not None and self.input_select is not js_undefined and \
                self.json_select is not None and self.json_select is not js_undefined:
            if self.onButtonClick is not None and self.onButtonClick is not js_undefined:
                jQuery(
                    "#phanterpwa-container-input-text-{0} .phanterpwa_input_icon_button".format(self.input_select)
                ).off(
                    "click.phanterpwa_input_icon_button"
                ).on(
                    "click.phanterpwa_input_icon_button",
                    lambda: self.onButtonClick(self.element)
                )
            input_name = self.input_select
            json_select = self.json_select
            placeholder = self.placeholder
            label = json_select['label']
            dft = json_select['default']
            data = json_select['data']
            jQuery("#phanterpwa-container-input-text-{0} .select-wrapper".format(input_name)).remove()
            vselect = SELECT(
            )
            has_default = False
            for x in data:
                if dft == x[0]:
                    has_default = True
                    vselect.append(
                        OPTION(x[1], _value=x[0], _selected="selected")
                    )
                else:
                    vselect.append(
                        OPTION(x[1], _value=x[0])
                    )
            phanterpwa_form_validator = self.element.attr("phanterpwa_form_validator")
            disabled = None
            if phanterpwa_form_validator is not None and phanterpwa_form_validator is not js_undefined:
                ppwa_fv = JSON.parse(phanterpwa_form_validator)
                if "IS_NOT_EMPTY" in ppwa_fv:
                    disabled = "disabled"

            if has_default:
                self.element.val(dft)
                vselect.insert(
                    0,
                    OPTION(placeholder, _value="", _disabled=disabled)
                )
            else:
                vselect.insert(
                    0,
                    OPTION(placeholder, _value="", _selected="selected", _disabled=disabled)
                )
            self.element.after(vselect.xml())
            value = jQuery(self.element).val()
            if value is None or value is js_undefined or value == "":
                jQuery("#phanterpwa-container-input-text-{0}".format(input_name)).addClass('input_is_empty')
            else:
                jQuery("#phanterpwa-container-input-text-{0}".format(input_name)).removeClass('input_is_empty')
            jQuery("#phanterpwa-container-input-text-{0}".format(input_name)).addClass('has_select')
            jQuery(
                "#phanterpwa-container-input-text-{0} .input-field>label".format(input_name)
            ).removeAttr("for").text(label)
            jQuery('#phanterpwa-container-input-text-{0} select'.format(input_name)).formSelect()

            def add_on_input(el):
                value = jQuery(el).val()
                if value is None or value is js_undefined or value == "":
                    jQuery("#phanterpwa-container-input-text-{0}".format(input_name)).addClass('input_is_empty')
                else:
                    jQuery("#phanterpwa-container-input-text-{0}".format(input_name)).removeClass('input_is_empty')
                self.element.val(value).trigger("change")
            jQuery(
                '#phanterpwa-container-input-text-{0} .select-wrapper select'.format(input_name)
            ).off(
                'change.change_{0}'.format(input_name),
            ).on(
                'change.change_{0}'.format(input_name),
                lambda: add_on_input(this)
            )
        return self.element



import os
from ...helpers import DIV

PRELOADER = DIV(
    DIV(
        DIV(
            DIV(
                DIV(
                    DIV(_class="phanterpwa_run_points_one"),
                    _class="phanterpwa_run_points phanterpwa_run_points_one_content"),
                DIV(
                    DIV(_class="phanterpwa_run_points_two"),
                    _class="phanterpwa_run_points phanterpwa_run_points_two_content"),
                DIV(
                    DIV(_class="phanterpwa_run_points_three"),
                    _class="phanterpwa_run_points phanterpwa_run_points_three_content"),
                DIV(
                    DIV(_class="phanterpwa_run_points_four"),
                    _class="phanterpwa_run_points phanterpwa_run_points_four_content"),
                DIV(
                    DIV(_class="phanterpwa_run_points_five"),
                    _class="phanterpwa_run_points phanterpwa_run_points_five_content"),
                _class="pwa_run_points_container"
            ),
            _class='preloader-wrapper enabled'
        ),
        _class="phanterpwa-preloader-wrapper"),
    _class="phanterpwa-components-preloaders-run_points"
)

PRELOADER.sass_file(
    os.path.join(os.path.dirname(__file__), "run_points.sass")
)
PRELOADER.sass_vars = {
    'COLOR1': 'red',
    'COLOR2': 'blue',
    'COLOR3': 'yellow',
    'COLOR4': 'pink',
    'COLOR5': 'green',
    'BORDERRADIUS': '100%',
    'CONTAINERWIDTH': '100px'
}



import os
import urllib.request
import base64
import json
from passlib.hash import pbkdf2_sha512
from phanterpwa.backend.decorators import (
    check_client_token,
    check_url_token,
    check_public_csrf_token,
    check_private_csrf_token,
    check_user_token,
    requires_no_authentication
)
from phanterpwa.mail import MailSender
from phanterpwa.samples import (
    email_activation_code,
    email_password,
    email_two_factor_code
)
from phanterpwa.backend.dataforms import FieldsDALValidateDictArgs
from phanterpwa.tools import (
    humanize_seconds,
    temporary_password,
    interpolate,
    generate_activation_code,
    check_activation_code,
    user_agent_parse,
    checkbox_bool
)
from phanterpwa.backend.pydal.extra_validations import (
    PASSWORD_MATCH_WITH_HASH,
    VALID_PASSWORD
)
from phanterpwa.i18n import browser_language
from phanterpwa.third_parties.xss import xssescape as E
from phanterpwa.gallery.cutter import PhanterpwaGalleryCutter
from phanterpwa.gallery.integrationDAL import PhanterpwaGalleryUserImage
from tornado import (
    web
)
from phanterpwa.backend.security import (
    Serialize,
    SignatureExpired,
    BadSignature,
    URLSafeSerializer,
)
from pydal import Field
from pydal.validators import (
    IS_EQUAL_TO,
    IS_EMAIL
)
from datetime import (
    datetime,
    timedelta
)


class CAS(web.RequestHandler):
    """
        url: '/api/cas/'
    """

    def initialize(self, app_name, projectConfig, DALDatabase, Translator_email, i18nTranslator=None, logger_api=None):
        self.app_name = app_name
        self.projectConfig = projectConfig
        self.DALDatabase = DALDatabase
        self.Translator_email = Translator_email
        self.i18nTranslator = i18nTranslator
        if logger_api:
            self.logger_api = logger_api
        if i18nTranslator:
            self.T = i18nTranslator.T
        self.set_header("Access-Control-Allow-Origin", "*")
        self.set_header(
            "Access-Control-Allow-Headers",
            "".join([
                "phanterpwa-cas-authorization,"
                "cache-control"
            ])
        )
        self.set_header('Access-Control-Allow-Methods', 'GET, OPTIONS, POST, PUT, DELETE')
        if self.request.headers.get("phanterpwa-language"):
            self.phanterpwa_language = self.request.headers.get("phanterpwa-language")
        else:
            self.phanterpwa_language = browser_language(self.request.headers.get("Accept-Language"))
        if self.i18nTranslator:
            self.i18nTranslator.direct_translation = self.phanterpwa_language
        self.phanterpwa_user_agent = str(self.request.headers.get('User-Agent'))
        self.phanterpwa_remote_ip = self.request.headers.get("X-Real-IP") or \
            self.request.headers.get("X-Forwarded-For") or \
            self.request.remote_ip
        self.phanterpwa_form_identify = None

    def check_origin(self, origin):
        return True

    def _request_summary(self) -> str:
        client_ip = self.request.headers.get('X-Real-IP') or\
            self.request.headers.get('X-Forwarded-For', '').split(',')[0].strip() or\
            self.request.remote_ip
        summary = "{0} {1} ({2})".format(
            self.request.method,
            self.request.uri,
            client_ip,
        )
        if hasattr(self, "phanterpwa_current_user") and self.phanterpwa_current_user is not None:
            summary = "{0} {1} ({2} - {3})".format(
                self.request.method,
                self.request.uri,
                client_ip,
                self.phanterpwa_current_user.email
            )
        return summary

    def options(self, *args):
        self.set_status(200)
        self.write({"status": "OK"})

    @check_cas_token()
    def get(self):
        self.write({
            "status": "OK",
            "message": "Hello World!",
            "project": self.projectConfig["PROJECT"]
        })


    @check_public_csrf_token(form_identify=[
        "phanterpwa-form-login", "user_locked", "phanterpwa-form-request_password"])
    def post(self, *args):
        self.phanterpwa_authorization = self.request.headers.get('phanterpwa-authorization')
        dict_arguments = {k: self.request.arguments.get(k)[0].decode('utf-8') for k in self.request.arguments}

        if dict_arguments.get('edata'):
            used_temporary = None
            edata = dict_arguments['edata']
            email, password = edata.split(":")
            email = base64.b64decode(email).decode('utf-8').strip().lower()
            password = base64.b64decode(password).decode('utf-8')
            q_user = self.DALDatabase(self.DALDatabase.auth_user.email == email).select().first()
            if q_user:
                if not q_user.login_attempts:
                    q_user.update_record(login_attempts=1)
                else:
                    q_user.update_record(login_attempts=q_user.login_attempts + 1)
                result = None
                try:
                    result = pbkdf2_sha512.verify(
                        "password{0}{1}".format(
                            password, self.projectConfig['BACKEND'][self.app_name]['secret_key']),
                        q_user.password_hash
                    )
                except Exception:
                    self.logger_api.error("Problem on check password", exc_info=True)
                finally:
                    if not result and\
                        q_user.temporary_password_expire and\
                        (datetime.now() < q_user.temporary_password_expire) and q_user.temporary_password_hash:
                        result = pbkdf2_sha512.verify(
                            "password{0}{1}".format(
                                password, self.projectConfig['BACKEND'][self.app_name]['secret_key']),
                            q_user.temporary_password_hash
                        )
                        if result:
                            used_temporary = password
                            q_user.update_record(login_attempts=0)
                if q_user.login_attempts > self.projectConfig['BACKEND'][self.app_name]['max_login_attempts']:
                    if q_user.datetime_next_attempt_to_login and\
                            (datetime.now() <= q_user.datetime_next_attempt_to_login):
                        t_delta = q_user.datetime_next_attempt_to_login - datetime.now()
                        msg = "Please, wait approximately {time_next_attempt} to try again."
                        tna = int(t_delta.total_seconds())
                        message = msg.format(time_next_attempt=humanize_seconds(tna))
                        message_i18n = self.T(msg).format(time_next_attempt=humanize_seconds(tna, self.i18nTranslator))
                        self.DALDatabase.commit()
                        self.set_status(400)
                        return self.write({
                            'status': 'Bad Request',
                            'code': 400,
                            'message': message,
                            'login_attempts': q_user.login_attempts,
                            'i18n': {
                                'message': message_i18n,
                            }
                        })
                    else:
                        q_user.update_record(login_attempts=1)
                if result:
                    remember_me = False
                    timeout_token_user = self.projectConfig['BACKEND'][self.app_name]['default_time_user_token_expire']
                    rem_me = checkbox_bool(dict_arguments['remember_me'])
                    if rem_me:
                        remember_me = True
                        timeout_token_user = self.projectConfig['BACKEND'][self.app_name]['default_time_user_token_expire_remember_me']
                    t_user = Serialize(
                        self.projectConfig['BACKEND'][self.app_name]['secret_key'],
                        timeout_token_user
                    )
                    content = {
                        'id': str(q_user.id),
                        'email': email
                    }
                    token_user = t_user.dumps(content)
                    q_role = self.DALDatabase(
                        (self.DALDatabase.auth_membership.auth_user == q_user.id) &
                        (self.DALDatabase.auth_group.id == self.DALDatabase.auth_membership.auth_group)
                    ).select(
                        self.DALDatabase.auth_group.id, self.DALDatabase.auth_group.role, orderby=self.DALDatabase.auth_group.grade
                    )
                    roles = [x.role for x in q_role]
                    dict_roles = {x.id: x.role for x in q_role}
                    roles_id = [x.id for x in q_role]
                    role = None
                    if roles:
                        role = roles[-1]
                    q_user.update_record(login_attempts=0)
                    t_client = Serialize(
                        self.projectConfig['BACKEND'][self.app_name]['secret_key'],
                        self.projectConfig['BACKEND'][self.app_name]['default_time_client_token_expire']
                    )
                    t_url = URLSafeSerializer(
                        self.projectConfig['BACKEND'][self.app_name]["secret_key"],
                        salt="url_secret_key"
                    )
                    r_client = self.DALDatabase(
                        self.DALDatabase.client.token == self.phanterpwa_client_token).select().first()
                    if r_client:
                        r_client.delete_record()
                    id_client = self.DALDatabase.client.insert(auth_user=q_user.id, date_created=datetime.now())
                    q_client = self.DALDatabase(self.DALDatabase.client.id == id_client).select().first()
                    content = {
                        'id_user': str(q_user.id),
                        'id_client': str(id_client),
                        'user_agent': self.phanterpwa_user_agent,
                        'remote_addr': self.phanterpwa_remote_ip
                    }

                    token_url = t_url.dumps(content)
                    token_client = t_client.dumps(content)
                    q_client.update_record(
                        token=token_client,
                        date_created=datetime.now(),
                        remember_me=remember_me,
                        locked=False,
                    )

                    if not q_user.permit_mult_login:
                        r_client = self.DALDatabase(
                            (self.DALDatabase.client.auth_user == q_user.id) &
                            (self.DALDatabase.client.token != token_client)
                        ).select()
                        if r_client:
                            r_client = self.DALDatabase(
                                (self.DALDatabase.client.auth_user == q_user.id) &
                                (self.DALDatabase.client.token != token_client)
                            ).delete()
                    self.DALDatabase.commit()
                    user_image = PhanterpwaGalleryUserImage(q_user.id, self.DALDatabase, self.projectConfig)

                    if (q_user.two_factor_login or two_factor) and not used_temporary and not self.phanterpwa_form_identify=="user_locked":
                        two_factor_serialize = URLSafeSerializer(
                            self.projectConfig['BACKEND'][self.app_name]["secret_key"],
                            salt="two_factor_url"
                        )
                        content = {
                            'id_user': str(q_user.id),
                            'id_client': str(id_client),
                            'user_agent': self.phanterpwa_user_agent,
                            'remote_addr': self.phanterpwa_remote_ip
                        }
                        two_factor_url = two_factor_serialize.dumps(content)
                        two_factor_code = generate_activation_code()
                        self.Translator_email.direct_translation = self.phanterpwa_language
                        keys_formatter = dict(
                            app_name=self.projectConfig['PROJECT']['name'],
                            user_name="{0} {1}".format(
                                q_user.first_name,
                                q_user.last_name
                            ),
                            code=two_factor_code,
                            time_expires=humanize_seconds(
                                self.projectConfig['BACKEND'][self.app_name]['default_time_two_factor_code_expire'],
                                self.Translator_email
                            ),
                            user_agent=self.phanterpwa_user_agent,
                            user_ip=self.phanterpwa_remote_ip,
                            copyright=interpolate(
                                self.projectConfig['CONTENT_EMAILS']['copyright'], {'now': datetime.now().year}),
                            link_to_your_page=self.projectConfig['CONTENT_EMAILS']['link_to_your_site']
                        )
                        email_password.text.formatter(keys_formatter)
                        text_email = email_two_factor_code.text.html(
                            minify=True,
                            translate=True,
                            formatter=keys_formatter,
                            i18nInstance=self.Translator_email,
                            dictionary=self.phanterpwa_language,
                            do_not_translate=["\n", " ", "\n\n", "&nbsp;"],
                            escape_string=False
                        )
                        html_email = email_two_factor_code.html.html(
                            minify=True,
                            translate=True,
                            formatter=keys_formatter,
                            i18nInstance=self.Translator_email,
                            dictionary=self.phanterpwa_language,
                            do_not_translate=["\n", " ", "\n\n", "&nbsp;"],
                            escape_string=False
                        )

                        e_mail = MailSender(
                            self.projectConfig['EMAIL']['default_sender'],
                            self.projectConfig['EMAIL']['password'],
                            q_user.email,
                            subject="Two factor authentication code",
                            text_message=text_email,
                            html_message=html_email,
                            server=self.projectConfig['EMAIL']['server'],
                            port=self.projectConfig['EMAIL']['port'],
                            use_tls=self.projectConfig['EMAIL']['use_tls'],
                            use_ssl=self.projectConfig['EMAIL']['use_ssl']
                        )
                        result = ""
                        try:
                            if self.projectConfig["PROJECT"]["debug"]:
                                self.logger_api.warning("TWO FACTOR CODE: {0}".format(two_factor_code))
                            else:
                                self.logger_api.warning("Email from '{0}' to '{1}' -> Activation Code: {2}".format(
                                    self.projectConfig['EMAIL']['default_sender'],
                                    q_user.email,
                                    two_factor_code
                                ))
                                e_mail.send()
                        except Exception as e:
                            result = "Email from '{0}' to '{1}' don't send! -> Error: {2} -> password: {3}".format(
                                self.projectConfig['EMAIL']['default_sender'],
                                dict_arguments['email'],
                                e,
                                two_factor_code
                            )
                            self.logger_api.error(result, exc_info=True)
                            message = "There was an error trying to send the email."
                            message_i18n = self.T("There was an error trying to send the email.")
                            self.DALDatabase.rollback()
                            self.set_status(400)
                            return self.write({
                                'status': 'Bad Request',
                                'code': 400,
                                'message': message,
                                'i18n': {'message': message_i18n}
                            })
                        else:

                            self.DALDatabase.two_factor_login.insert(
                                auth_user=q_user.id,
                                two_factor_url=two_factor_url,
                                two_factor_code=two_factor_code
                            )
                            message = 'A code has been sent to your email.'
                            message_i18n = self.T('A code has been sent to your email.')
                            self.DALDatabase.commit()
                            self.set_status(206)
                            return self.write({
                                'status': 'OK',
                                'message': message,
                                'client_token': token_client,
                                'as': self.phanterpwa_form_identify,
                                'auth_user': {
                                    'remember_me': q_client.remember_me
                                },
                                'i18n': {
                                    'message': message_i18n
                                },
                                'authorization_url': two_factor_url
                            })

                    else:
                        self.set_status(200)
                        return self.write({
                            'status': 'OK',
                            'code': 200,
                            'message': 'The user is logged',
                            'authorization': token_user,
                            'client_token': token_client,
                            'url_token': token_url,
                            'used_temporary': used_temporary,
                            'auth_user': {
                                'id': str(q_user.id),
                                'first_name': E(q_user.first_name),
                                'last_name': E(q_user.last_name),
                                'email': email,
                                'remember_me': q_client.remember_me,
                                'roles': roles,
                                'role': role,
                                'dict_roles': dict_roles,
                                'roles_id': roles_id,
                                'activated': q_user.activated,
                                'image': user_image.id_image,
                                'two_factor': q_user.two_factor_login,
                                'multiple_login': q_user.permit_mult_login,
                                'locale': q_user.locale,
                                'social_login': None
                            },
                            'i18n': {
                                'message': self.T('The user is logged'),
                                'auth_user': {'role': self.T(role)}
                            }
                        })
                else:
                    q_user.update_record(
                        temporary_password_expire=datetime.now() +
                            timedelta(seconds=self.projectConfig['BACKEND'][self.app_name]['default_time_temporary_password_expire']),
                        datetime_next_attempt_to_login=datetime.now() +
                            timedelta(seconds=self.projectConfig['BACKEND'][self.app_name]['timeout_to_next_login_attempt'])
                    )
                    msg = 'Wrong password! Attempt {attempt_number} from {max_attempts}'
                    message = msg.format(
                        attempt_number=q_user.login_attempts,
                        max_attempts=self.projectConfig['BACKEND'][self.app_name]['max_login_attempts']
                    )
                    message_i18n = self.T(msg).format(
                        attempt_number=q_user.login_attempts,
                        max_attempts=self.projectConfig['BACKEND'][self.app_name]['max_login_attempts']
                    )
                    self.DALDatabase.commit()
                    self.set_status(400)
                    return self.write({
                        'status': 'Bad Request',
                        'code': 400,
                        'message': message,
                        'login_attempts': q_user.login_attempts,
                        'i18n': {
                            'message': message_i18n
                        }
                    })
            else:
                self.set_status(401)
                return self.write({
                    'status': 'Unauthorized',
                    'code': 401,
                    'message': 'Invalid password or email',
                    'i18n': {
                        'message': self.T('Invalid password or email')
                    }
                })
        self.set_status(400)
        return self.write({
            'status': 'Bad Request',
            'code': 400,
            'message': 'Invalid password or email',
            'as': dict_arguments.get('edata'),
            'i18n': {
                'message': self.T('Invalid password or email')
            }
        })

    @check_client_token()
    @check_user_token()
    def delete(self, *args):
        id_session = args[0]
        t_client = Serialize(
            self.projectConfig['BACKEND'][self.app_name]['secret_key'],
            self.projectConfig['BACKEND'][self.app_name]['default_time_client_token_expire']
        )
        if self.phanterpwa_client_token:
            db = self.DALDatabase
            # self.phanterpwa_current_client.as_dict(datetime_to_str=True)
            q = db(db.client.token == self.phanterpwa_client_token).select().first()
            if q:
                token_content_client = None
                try:
                    token_content_client = t_client.loads(self.phanterpwa_client_token)
                except BadSignature:
                    msg = 'The client have a invalid client-token, a new one has been generated.'
                    token_content_client = None
                except SignatureExpired:
                    msg = 'The client have a expired client-token, a new one has been generated.'
                    token_content_client = None
                if token_content_client:
                    sessions = []
                    q_sessions = db(db.client.auth_user==self.phanterpwa_current_user.id).select(orderby=db.client.date_created)
                    for x in q_sessions:
                        this_session = False
                        if x.token == self.phanterpwa_client_token:
                            this_session = True
                        if (str(x.id) == id_session) and not this_session:
                            x.delete_record()
                        else:
                            tc = None
                            try:
                                tc = t_client.loads(x.token)
                            except:
                                tc = None
                            user_agent = None
                            remote_addr = None
                            date_created = x.date_created
                            if tc:
                                remote_addr = tc['remote_addr']
                                user_agent = tc['user_agent']
                            sessions.append(
                                dict(
                                    user_agent=user_agent,
                                    agent = user_agent_parse(user_agent),
                                    remote_addr=remote_addr,
                                    date_created=str(date_created),
                                    this_session=this_session,
                                    identify=x.id
                                )
                            )

                    self.set_status(200)
                    self.write({
                        'status': 'OK',
                        'code': 200,
                        'message': 'Deleted Session',
                        'i18n': {
                            'message': self.T('Deleted Session')
                        },
                        'sessions': sessions
                    })




import os
import glob
from pydal import DAL
from core import (projectConfig, logger_api)
from phanterpwa.backend.pydal.auth import AuthTables
from phanterpwa.backend.pydal.credentials import CredentialsTables
from phanterpwa.backend.pydal.gallery import GalleryTables
database_path = os.path.join(os.path.dirname(__file__), '..', 'databases')
files = glob.glob(os.path.join(os.path.dirname(__file__), "*.py"))

db = DAL(
    'sqlite://storage.sqlite',
    pool_size=10,
    folder=database_path,
    # migrate_enabled=True,
    check_reserved=['all']
)

authdb = AuthTables(projectConfig, db, logger_api=logger_api)
credentialsdb = CredentialsTables(projectConfig, db, logger_api=logger_api)
phantergallerydb = GalleryTables(db)

__all__ = ['authdb', 'credentialsdb', 'phantergallerydb']
_all = [os.path.basename(x)[0:-3] for x in files if os.path.basename(x) != "__init__.py" and x[-3:] == ".py"]
_all.sort()
for y in _all:
    __all__.append(y)
__all__.append("db")



from phanterpwa.frontend import helpers
from org.transcrypt.stubs.browser import __pragma__
__pragma__('alias', "jQuery", "$")
__pragma__('skip')

# it is ignored on transcrypt
window = jQuery = console = document = localStorage = URL =\
    sessionStorage = this = FileReader = JSON = js_undefined = navigator = __new__ = Date = 0

__pragma__('noskip')


__pragma__('kwargs')


I18N = helpers.I18N
class ApiServer():

    def __init__(self):
        self.remote_address = window.PhanterPWA.CONFIG["APP"]["http_address"]

    @staticmethod
    def _process_args(args):
        s_args = ""
        ISINSTANCEOF = __pragma__("js", "{}", "args instanceof Array")
        if isinstance(args, list) or ISINSTANCEOF:
            s_args = "/".join(args)
        if s_args != "":
            s_args = "{0}/".format(s_args)
        return s_args

    @staticmethod
    def _process_parameters(parameters):
        _vars = dict()
        for x in parameters.keys():
            if x == "_":
                _vars[x] = parameters[x]
            elif x.startswith("_") and len(x) > 1:
                _vars[x[1:]] = parameters[x]
        if len(_vars.keys()) > 0:
            return _vars
        else:
            return parameters.get("url_vars", None)

    @staticmethod
    def _serialize_vars(_vars):
        ISTYPEOF = __pragma__("js", "{}", "_vars instanceof FormData")
        if _vars is None or _vars is js_undefined:
            return ""
        elif ISTYPEOF:
            return ""
        elif isinstance(_vars, dict):
            __pragma__('jsiter')
            jsdict = {}
            __pragma__('nojsiter')
            has_value = False
            for x in _vars.keys():
                has_value = True
                if _vars[x] is not None and _vars[x] is not js_undefined:
                    jsdict[x] = _vars[x]
            if has_value:
                return "?{0}".format(jQuery.param(jsdict))
            else:
                return ""
        else:
            t = jQuery.param(_vars)
            if t is not None or t is not js_undefined:
                return "?{0}".format(t)
            else:
                return ""

    def GET(self, *args, **parameters):
        url_vars = None
        onComplete = None
        headers = {
            'Cache-Control': 'no-store, must-revalidate, no-cache, max-age=0',
            'phanterpwa-application': window.PhanterPWA.CONFIG['PROJECT']['name'],
            'phanterpwa-application-version': window.PhanterPWA.CONFIG['PROJECT']['version']
        }
        reload_phanterpwa = parameters.get("reload", True)
        url_args = parameters.get("url_args", None)
        if len(args) > 0:
            url_args = args
        url_vars = self._process_parameters(parameters)
        onComplete = parameters.get("onComplete", lambda: console.info("GET") if window.PhanterPWA.DEBUG else "")
        onError = parameters.get("onError", None)
        timeout = parameters.get("timeout", None)
        if "headers" in parameters:
            if parameters["headers"] is not None:
                for x in parameters["headers"]:
                    headers[x] = parameters["headers"][x]
        date_stamp = __new__(Date().getTime())
        window.PhanterPWA.ProgressBar.addEventProgressBar("GET_" + date_stamp)
        client_token = localStorage.getItem("phanterpwa-client-token")
        authorization = sessionStorage.getItem("phanterpwa-authorization")
        all_args = self._process_args(url_args)
        all_vars = self._serialize_vars(url_vars)
        current_uri = "/{0}{1}".format(all_args, all_vars)
        url = "{0}{1}".format(self.remote_address, current_uri)
        if "get_cache" in parameters:
            if callable(parameters["get_cache"]):
                get_cache = parameters["get_cache"]
                if current_uri in window.PhanterPWA.Cache:
                    get_cache(window.PhanterPWA.Cache[current_uri])

        def _after_sucess(data):
            if reload_phanterpwa:
                window.PhanterPWA.reload()
            data_hash = data.hash
            data_uri = data.uri
            if data_hash is not js_undefined and data_uri is not js_undefined:
                if data_uri not in window.PhanterPWA.Cache:
                    window.PhanterPWA.Cache[data_uri] = data
                else:
                    if data_hash != window.PhanterPWA.Cache[data_uri].hash:
                        window.PhanterPWA.Cache[data_uri] = data

            window.PhanterPWA.ProgressBar.removeEventProgressBar("GET_" + date_stamp)

        def _after_error(data, ajax_status):
            window.PhanterPWA.ProgressBar.removeEventProgressBar("GET_" + date_stamp)
            self.on_ajax_error(data, ajax_status, onError)

        __pragma__('jsiter')
        ajax_param = {
            'url': url,
            'type': "GET",
            'complete': lambda a, b, c: onComplete(a, b, c),
            'success': _after_sucess,
            'error': _after_error,
            'datatype': 'json',
            'crossDomain': True,
            'headers': headers,
        }
        if timeout is not None:
            ajax_param["timeout"] = timeout
        if client_token is not None:
            ajax_param['headers']['phanterpwa-client-token'] = client_token
        if authorization is not None and authorization is not js_undefined:
            ajax_param['headers']['phanterpwa-authorization'] = authorization
        else:
            authorization = localStorage.getItem("phanterpwa-authorization")
            if authorization is not None and authorization is not js_undefined:
                ajax_param['headers']['phanterpwa-authorization'] = authorization
        __pragma__('nojsiter')
        jQuery.ajax(ajax_param)

    def DELETE(self, *args, **parameters):
        url_vars = None
        onComplete = None
        headers = {
            'Cache-Control': 'no-store, must-revalidate, no-cache, max-age=0',
            'phanterpwa-application': window.PhanterPWA.CONFIG['PROJECT']['name'],
            'phanterpwa-application-version': window.PhanterPWA.CONFIG['PROJECT']['version']
        }
        reload_phanterpwa = parameters.get("reload", True)
        url_args = parameters.get("url_args", None)
        if len(args) > 0:
            url_args = args
        url_vars = self._process_parameters(parameters)
        onComplete = parameters.get("onComplete", lambda: console.info("DELETE") if window.PhanterPWA.DEBUG else "")
        onError = parameters.get("onError", None)
        timeout = parameters.get("timeout", None)
        if "headers" in parameters:
            if parameters["headers"] is not None:
                for x in parameters["headers"]:
                    headers[x] = parameters["headers"][x]
        date_stamp = __new__(Date().getTime())
        window.PhanterPWA.ProgressBar.addEventProgressBar("DELETE_" + date_stamp)
        client_token = localStorage.getItem("phanterpwa-client-token")
        authorization = sessionStorage.getItem("phanterpwa-authorization")
        url = "{0}/{1}{2}".format(self.remote_address, self._process_args(url_args), self._serialize_vars(url_vars))
        def _after_error(data, ajax_status):
            window.PhanterPWA.ProgressBar.removeEventProgressBar("DELETE_" + date_stamp)
            self.on_ajax_error(data, ajax_status, onError)
        __pragma__('jsiter')
        ajax_param = {
            'url': url,
            'type': "DELETE",
            'complete': lambda a, b, c: (onComplete(a, b, c), window.PhanterPWA.reload() if reload_phanterpwa else None),
            'success': lambda: window.PhanterPWA.ProgressBar.removeEventProgressBar("DELETE_" + date_stamp),
            'error': _after_error,
            'datatype': 'json',
            'crossDomain': True,
            'headers': headers
        }
        if timeout is not None:
            ajax_param["timeout"] = timeout
        if client_token is not None:
            ajax_param['headers']['phanterpwa-client-token'] = client_token
        if authorization is not None and authorization is not js_undefined:
            ajax_param['headers']['phanterpwa-authorization'] = authorization
        else:
            authorization = localStorage.getItem("phanterpwa-authorization")
            if authorization is not None and authorization is not js_undefined:
                ajax_param['headers']['phanterpwa-authorization'] = authorization
        __pragma__('nojsiter')
        jQuery.ajax(ajax_param)

    def POST(self, *args, **parameters):
        form_data = None
        onComplete = None
        headers = {
            'Cache-Control': 'no-store, must-revalidate, no-cache, max-age=0',
            'phanterpwa-application': window.PhanterPWA.CONFIG['PROJECT']['name'],
            'phanterpwa-application-version': window.PhanterPWA.CONFIG['PROJECT']['version']
        }
        reload_phanterpwa = parameters.get("reload", True)
        url_args = parameters.get("url_args", None)
        if len(args) > 0:
            url_args = args
        if "form_data" in parameters:
            form_data = parameters["form_data"]
        onComplete = parameters.get("onComplete", lambda: console.info("POST") if window.PhanterPWA.DEBUG else "")
        onError = parameters.get("onError", None)
        timeout = parameters.get("timeout", None)
        if "headers" in parameters:
            if parameters["headers"] is not None:
                for x in parameters["headers"]:
                    headers[x] = parameters["headers"][x]
        date_stamp = __new__(Date().getTime())
        window.PhanterPWA.ProgressBar.addEventProgressBar("POST_" + date_stamp)
        client_token = localStorage.getItem("phanterpwa-client-token")
        authorization = sessionStorage.getItem("phanterpwa-authorization")
        url = "{0}/{1}".format(self.remote_address, self._process_args(url_args))
        def _after_error(data, ajax_status):
            window.PhanterPWA.ProgressBar.removeEventProgressBar("POST_" + date_stamp)
            self.on_ajax_error(data, ajax_status, onError)
        __pragma__('jsiter')
        ajax_param = {
            'url': url,
            'type': "POST",
            'data': form_data,
            'complete': lambda a, b, c: (onComplete(a, b, c), window.PhanterPWA.reload() if reload_phanterpwa else None),
            'success': lambda: window.PhanterPWA.ProgressBar.removeEventProgressBar("POST_" + date_stamp),
            'error': _after_error,
            'datatype': 'json',
            'crossDomain': True,
            'headers': headers
        }
        if timeout is not None:
            ajax_param["timeout"] = timeout
        if client_token is not None:
            ajax_param['headers']['phanterpwa-client-token'] = client_token
        if authorization is not None and authorization is not js_undefined:
            ajax_param['headers']['phanterpwa-authorization'] = authorization
        else:
            authorization = localStorage.getItem("phanterpwa-authorization")
            if authorization is not None and authorization is not js_undefined:
                ajax_param['headers']['phanterpwa-authorization'] = authorization
        ISTYPEOF = __pragma__("js", "{}", "form_data instanceof FormData")
        if ISTYPEOF:
            ajax_param['processData'] = False
            ajax_param['contentType'] = False
        __pragma__('nojsiter')
        jQuery.ajax(ajax_param)

    def PUT(self, *args, **parameters):
        form_data = None
        onComplete = None
        headers = {
            'Cache-Control': 'no-store, must-revalidate, no-cache, max-age=0',
            'phanterpwa-application': window.PhanterPWA.CONFIG['PROJECT']['name'],
            'phanterpwa-application-version': window.PhanterPWA.CONFIG['PROJECT']['version']
        }
        reload_phanterpwa = parameters.get("reload", True)
        url_args = parameters.get("url_args", None)
        if len(args) > 0:
            url_args = args
        if "form_data" in parameters:
            form_data = parameters["form_data"]
        onComplete = parameters.get("onComplete", lambda: console.info("PUT") if window.PhanterPWA.DEBUG else "")
        onError = parameters.get("onError", None)
        timeout = parameters.get("timeout", None)
        if "headers" in parameters:
            if parameters["headers"] is not None:
                for x in parameters["headers"]:
                    headers[x] = parameters["headers"][x]
        date_stamp = __new__(Date().getTime())
        window.PhanterPWA.ProgressBar.addEventProgressBar("PUT_" + date_stamp)
        client_token = localStorage.getItem("phanterpwa-client-token")
        authorization = sessionStorage.getItem("phanterpwa-authorization")
        url = "{0}/{1}".format(self.remote_address, self._process_args(url_args))
        def _after_error(data, ajax_status):
            window.PhanterPWA.ProgressBar.removeEventProgressBar("PUT_" + date_stamp)
            self.on_ajax_error(data, ajax_status, onError)
        __pragma__('jsiter')
        ajax_param = {
            'url': url,
            'type': "PUT",
            'data': form_data,
            'complete': lambda a, b, c: (onComplete(a, b, c), window.PhanterPWA.reload() if reload_phanterpwa else None),
            'success': lambda: window.PhanterPWA.ProgressBar.removeEventProgressBar("PUT_" + date_stamp),
            'error': _after_error,
            'datatype': 'json',
            'crossDomain': True,
            'headers': headers
        }
        if timeout is not None:
            ajax_param["timeout"] = timeout
        if client_token is not None:
            ajax_param['headers']['phanterpwa-client-token'] = client_token
        if authorization is not None and authorization is not js_undefined:
            ajax_param['headers']['phanterpwa-authorization'] = authorization
        else:
            authorization = localStorage.getItem("phanterpwa-authorization")
            if authorization is not None and authorization is not js_undefined:
                ajax_param['headers']['phanterpwa-authorization'] = authorization
        ISTYPEOF = __pragma__("js", "{}", "form_data instanceof FormData")
        if ISTYPEOF:
            ajax_param['processData'] = False
            ajax_param['contentType'] = False
        __pragma__('nojsiter')
        jQuery.ajax(ajax_param)

    def getClientToken(self, callback=None):
        def onComplete(data, ajax_status):
            if ajax_status == "success":
                auth_user = data.responseJSON.auth_user
                if (auth_user == "anonymous") or (auth_user == "logout"):
                    sessionStorage.removeItem("phanterpwa-authorization")
                    sessionStorage.removeItem("auth_user")
                    localStorage.removeItem("phanterpwa-authorization")
                    localStorage.removeItem("auth_user")
                client_token = data.responseJSON.client_token
                url_token = data.responseJSON.url_token
                if (url_token is not js_undefined):
                    localStorage.setItem('phanterpwa-url-token', url_token)

                if (client_token is not js_undefined):
                    localStorage.setItem("phanterpwa-client-token", client_token)
                    if callback is not None and callback is not js_undefined:
                        callback(data, ajax_status)
            else:
                if data.status == 0:
                    console.info("Server Problem!")
                elif data.status == 400:
                    sessionStorage.js_clear()
                    localStorage.js_clear()

        client_token = localStorage.getItem("phanterpwa-client-token")
        session_authorization = sessionStorage.getItem("phanterpwa-authorization")

        if client_token is None or client_token is js_undefined:
            sessionStorage.removeItem("phanterpwa-authorization")
            sessionStorage.removeItem("auth_user")
        if session_authorization is None or session_authorization is js_undefined:
            local_authorization = localStorage.getItem("phanterpwa-authorization")
            if local_authorization is None or local_authorization is js_undefined:
                sessionStorage.removeItem("phanterpwa-authorization")
                sessionStorage.removeItem("auth_user")
                localStorage.removeItem("phanterpwa-authorization")
                localStorage.removeItem("auth_user")

        return self.GET(**{'url_args': ['api', 'client'], 'onComplete': onComplete})

    def reSignCredentials(self):
        date_stamp = __new__(Date().getTime())
        last_resing = sessionStorage.getItem("last_resing")
        authorization = sessionStorage.getItem("phanterpwa-authorization")
        auth_user = sessionStorage.getItem("auth_user")
        if auth_user is None or auth_user is js_undefined:
            auth_user = localStorage.getItem("auth_user")
        lets_go_resign = False
        if last_resing is not None and last_resing is not js_undefined:
            timeout_last_resign = window.PhanterPWA.CONFIG['CONFIGJS']['timeout_to_resign'] * 1000
            if (int(last_resing) + int(timeout_last_resign)) < int(date_stamp):
                lets_go_resign = True
                sessionStorage.setItem("last_resing", date_stamp)
        else:
            sessionStorage.setItem("last_resing", date_stamp)
            lets_go_resign = True

        def onComplete(data, ajax_status):
            json = data.responseJSON
            if ajax_status == "success":
                if json.url_token is not None and json.url_token is not js_undefined:
                    localStorage.setItem('phanterpwa-url-token', json.url_token)
                if json.client_token is not None and json.client_token is not js_undefined:
                    localStorage.setItem('phanterpwa-client-token', json.client_token)
                if json.authorization is not None and json.authorization is not js_undefined:
                    if auth_user["remember_me"] is True:
                        localStorage.setItem('phanterpwa-authorization', json.authorization)
                    else:
                        localStorage.setItem('phanterpwa-authorization', json.authorization)
        if lets_go_resign and authorization is not None and authorization is not js_undefined:
            if auth_user is not None and auth_user is not js_undefined:
                self.GET(**{
                    "url_args": ["api", "resigncredentials"],
                    "url_vars": {"_": date_stamp},
                    "onComplete": onComplete
                })

    def on_ajax_error(self, data, status, onError):
        message = I18N("Unexpected error!", **{"_pt-br": "Erro inesperado!"})
        json = data.responseJSON

        if data.status == 502:
            message = I18N(
                "Connection interrupted during the process.",
                **{"_pt-br": "Conexão interrompida durante o processo."}
            )
        elif data.status == 504:
            message = I18N(
                "Server busy, please try again later.",
                **{"_pt-br": "Servidor ocupado, tente mais tarde."}
            )
        elif data.status >= 500:
            message = I18N(
                "Server error (non-JSON response)",
                **{"_pt-br": "Erro do servidor (resposta não-JSON)"}
            )
        elif data.status == 400:
            message = I18N(
                "Bad request",
                **{"_pt-br": "Requisição inválida"}
            )
        elif data.status == 401:
            message = I18N(
                "Authentication is required.",
                **{"_pt-br": "É Necessário autenticar-se."}
            )
        elif data.status == 403:
            message = I18N(
                "Your credentials don't have enough privileges.",
                **{"_pt-br": "Sua credencial não tem privilégios suficiente."}
            )
        elif data.status == 404:
            message = I18N(
                "Resource not found",
                **{"_pt-br": "Recurso não encontrado"}
            )
        if json is not None and json is not js_undefined:

            reasons = None
            if json.i18n is not None and json.i18n is not js_undefined:
                if json.i18n.message is not None and json.i18n is not js_undefined:
                    message = json.i18n.message
                if json.i18n.reasons is not None and json.i18n is not js_undefined:
                    reasons = json.i18n.reasons
            else:
                if json.message is not None and json.message is not js_undefined:
                    message = json.message
                if json.reasons is not None and json.reasons is not js_undefined:
                    reasons = json.reasons
            if data.status == 401 or data.status == 403:
                if window.PhanterPWA.logged():
                    if data.status == 401:
                        if json.specification == "client deleted":
                            self.getClientToken(
                                callback=lambda: (
                                    window.PhanterPWA.reload_component("auth_user"),
                                    window.PhanterPWA.reload_component("left_bar")
                                )
                            )
                    window.PhanterPWA.open_code_way(data.status, window.PhanterPWA.Request, window.PhanterPWA.Response, reasons)

        elif status == "timeout" or data.status == 0:
            if status == "timeout":
                message = I18N("Server busy, please try again later..", **{"_pt-br": "Servidor ocupado, tente mais tarde."})
            else:
                message = I18N("Connection problem. Please try again.", **{"_pt-br": "Problema de conexão. Tente novamente."})

        if callable(onError):
            onError(data, status)
        elif message is not None:
            window.PhanterPWA.flash(**{'html': message})



__pragma__('nokwargs')



from org.transcrypt.stubs.browser import __pragma__
from phanterpwa.frontend.fmasks import stringFilter
__pragma__('alias', "jQuery", "$")
__pragma__('skip')

# it is ignored on transcrypt
window = jQuery = console = document = localStorage = RegExp = \
    sessionStorage = this = FileReader = JSON = js_undefined = navigator = __new__ = Date = 0

__pragma__('noskip')

validators_list = {
    "IS_NOT_EMPTY",
    "IS_DATE",
    "IS_EQUALS",
    "IS_ACTIVATION_CODE",
    "IS_EMAIL",
}

__pragma__('kwargs')


def zfill(number, size):
    number = int(number)
    number = str(number)
    s = number
    for x in range(size - len(number)):
        s = "0" + s
    return s


def check_activation_code(code, size=6):
    if isinstance(code, str):
        code = code.strip()
        xsize = size + 2
        if (len(code) == xsize) and "-" in code:
            cod, dig = code.split("-")
            ver = "ABCDEFGHI"
            su = 0
            for char in cod:
                su += int(char)
                if su >= 9:
                    su = su - 9
            if ver[su] == dig:
                return code
    return None


def format_iso_date_datetime(dvalue, dformat_out, dtype='datetime'):
    dformat_in = "yyyy-MM-dd"
    if dtype == "datetime":
        dformat_in = "yyyy-MM-ddTHH:mm:ss"
    if dvalue is "":
        return None
    elif len(str(dvalue)) != len(dformat_in):
        console.error("The date/datetime value is invalid")
        return None
    day = None
    month = None
    year = None
    hour = "00"
    minute = "00"
    second = "00"

    ini = dformat_in.indexOf("dd")
    day = dvalue[ini:ini + 2]
    ini = dformat_in.indexOf("MM")
    month = dvalue[ini:ini + 2]
    ini = dformat_in.indexOf("yyyy")
    year = dvalue[ini:ini + 4]
    if dtype == "datetime":
        ini = dformat_in.indexOf("HH")
        hour = dvalue[ini:ini + 2]
        ini = dformat_in.indexOf("mm")
        minute = dvalue[ini:ini + 2]
        ini = dformat_in.indexOf("ss")
        second = dvalue[ini:ini + 2]
    cdate = __new__(Date("{0}-{1}-{2}T{3}:{4}:{5}".format(year, month, day, hour, minute, second)))

    result = False
    if cdate.toJSON() is not None:
        result = True
    if result:
        dformat_out = dformat_out.replace("dd", zfill(day, 2))
        dformat_out = dformat_out.replace("MM", zfill(month, 2))
        dformat_out = dformat_out.replace("yyyy", zfill(year, 4))
        if dtype == "datetime":
            dformat_out = dformat_out.replace("HH", zfill(hour, 2))
            dformat_out = dformat_out.replace("mm", zfill(minute, 2))
            dformat_out = dformat_out.replace("ss", zfill(second, 2))
            dformat_out = dformat_out.replace("T", " ")
        return dformat_out
    else:
        console.error("The date/datetime value is invalid")
        return None


def check_datetime(dvalue, dformat="yyyy-MM-ddTHH:mm:ss", dtype="datetime"):
    if any(["yyyy" not in dformat,
            "MM" not in dformat,
            "dd" not in dformat]):
        return False
    if dtype == "datetime":
        if any(["HH" not in dformat,
                "mm" not in dformat,
                "ss" not in dformat]):
            return False
    if len(str(dvalue)) != len(dformat):
        return False

    day = None
    month = None
    year = None
    hour = "00"
    minute = "00"
    second = "00"

    ini = dformat.indexOf("dd")
    day = dvalue[ini:ini + 2]
    ini = dformat.indexOf("MM")
    month = dvalue[ini:ini + 2]
    ini = dformat.indexOf("yyyy")
    year = dvalue[ini:ini + 4]
    if dtype == "datetime":
        ini = dformat.indexOf("HH")
        hour = dvalue[ini:ini + 2]
        ini = dformat.indexOf("mm")
        minute = dvalue[ini:ini + 2]
        ini = dformat.indexOf("ss")
        second = dvalue[ini:ini + 2]
    cdate = __new__(Date("{0}-{1}-{2}T{3}:{4}:{5}".format(year, month, day, hour, minute, second)))

    result = False
    if cdate.toJSON() is not None:
        result = True
    return result


class Valider():
    def __init__(self, value, validators_list):
        self.validators_list = validators_list
        self.value = value
        self.tests = {}

    def validate(self):
        validate_test_pass = list()
        validate_test = self.validators_list
        value_for_validate = self.value
        is_empty_or = False
        self.tests = {}
        for x in validate_test:
            self.tests[x] = "Ignored"
        if "IS_EMPTY_OR" in validate_test:
            if (value_for_validate is js_undefined) or \
                (value_for_validate is None) or (value_for_validate == ""):
                validate_test_pass.append(True)
                self.tests["IS_EMPTY_OR"] = "Pass"
                is_empty_or = True
            else:
                self.tests["IS_EMPTY_OR"] = "Fail"
            validate_test.pop("IS_EMPTY_OR")
        if not is_empty_or:
            for x in validate_test:
                if x is not None and x is not js_undefined:
                    validate_test_pass.append(self._validates(x))

        if all(validate_test_pass):
            return True

        else:
            return False

    def _validates(self, validate_name):
        value_for_validate = self.value
        validate_test_pass = list()
        is_not_valid = True
        if validate_name.startswith("IS_IN_SET:"):
            is_not_valid = False
            res = False
            list_options = JSON.parse(validate_name[10:])
            if list_options is not None or list_options is not js_undefined:
                list_options = JSON.parse(list_options)
                new_list_options = [str(x[1]) for x in list_options]
                if new_list_options.indexOf(value_for_validate) > -1:
                    res = True
            validate_test_pass.append(res)

        if validate_name == "IS_NOT_EMPTY":
            self.tests[validate_name] = "Fail"
            is_not_valid = False
            if (value_for_validate is js_undefined) or \
                (value_for_validate is None) or (value_for_validate == ""):
                validate_test_pass.append(False)
            else:
                validate_test_pass.append(True)
                self.tests[validate_name] = "Pass"

        if validate_name == "IS_ACTIVATION_CODE":
            self.tests[validate_name] = "Fail"
            is_not_valid = False
            is_activation_code = False
            res = check_activation_code(value_for_validate)
            if res is not None:
                is_activation_code = True
            validate_test_pass.append(is_activation_code)

        if validate_name.startswith("IS_DATE"):
            self.tests[validate_name] = "Fail"
            is_not_valid = False
            if validate_name.startswith("IS_DATE:"):
                dformat = validate_name[8:]
                res = check_datetime(value_for_validate, dformat, "date")
                validate_test_pass.append(res)
            elif validate_name.startswith("IS_DATETIME:"):
                dformat = validate_name[12:]
                res = check_datetime(value_for_validate, dformat, "datetime")
                validate_test_pass.append(res)
            elif validate_name == "IS_DATETIME":
                dformat = validate_name[12:]
                res = check_datetime(value_for_validate)
                validate_test_pass.append(res)
            else:
                dformat = validate_name[12:]
                res = check_datetime(value_for_validate, "yyyy-MM-dd", "date")
                validate_test_pass.append(res)

        if validate_name.startswith("IS_EQUALS:"):
            self.tests[validate_name] = "Fail"
            is_not_valid = False
            comp = validate_name[10:]
            if comp.startswith("#"):
                val = jQuery(comp).val()
                if val == value_for_validate:
                    validate_test_pass.append(True)
                    self.tests[validate_name] = "Pass"
                else:
                    validate_test_pass.append(False)
            else:
                if comp == value_for_validate:
                    validate_test_pass.append(True)
                    self.tests[validate_name] = "Pass"
                else:
                    validate_test_pass.append(False)

        if validate_name.startswith("MATCH:"):
            self.tests[validate_name] = "Fail"
            is_not_valid = False
            regex = __new__(RegExp(validate_name[6:]))
            if value_for_validate.match(regex) is not None:
                validate_test_pass.append(True)
                self.tests[validate_name] = "Pass"
            else:
                validate_test_pass.append(False)

        if validate_name.startswith("MASK:"):
            self.tests[validate_name] = "Fail"
            is_not_valid = False
            mask_val = validate_name[5:]
            cont_mask = 0
            for x in str(mask_val):
                if x == "#":
                    cont_mask += 1

            if len(stringFilter(str(value_for_validate))) == cont_mask:
                validate_test_pass.append(True)
                self.tests[validate_name] = "Pass"
            else:
                validate_test_pass.append(False)
        if validate_name == "IS_EMAIL":
            value_for_validate = str(value_for_validate).strip()
            self.tests[validate_name] = "Fail"
            is_not_valid = False
            if "@" in value_for_validate:
                REGEX_BODY = __pragma__(
                    'js',
                    '{}',
                    r'/^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}])|(([_a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/'
                )
                if REGEX_BODY.test(value_for_validate):
                    validate_test_pass.append(True)
                    self.tests[validate_name] = "Pass"
                else:
                    validate_test_pass.append(False)
            else:
                validate_test_pass.append(False)

        if is_not_valid:
            console.error("The {0} is not valid!".format(validate_name))
            return False

        if all(validate_test_pass):
            return True
        else:
            return False



__pragma__('nokwargs')



import phanterpwa.frontend.gatehandler as gatehandler
import phanterpwa.frontend.helpers as helpers
import phanterpwa.frontend.components.widgets as widgets
import phanterpwa.frontend.components.left_bar as left_bar
from org.transcrypt.stubs.browser import __pragma__

__pragma__('alias', "jQuery", "$")
__pragma__('skip')
jQuery = sessionStorage = JSON = M = js_undefined = window = setTimeout = \
    __new__ = FormData = console = localStorage = document = this = 0
__pragma__('noskip')

DIV = helpers.XmlConstructor.tagger("div")
I = helpers.XmlConstructor.tagger("i")
H1 = helpers.XmlConstructor.tagger("h1")
H2 = helpers.XmlConstructor.tagger("h2")
H3 = helpers.XmlConstructor.tagger("h3")
STRONG = helpers.XmlConstructor.tagger("strong")
SPAN = helpers.XmlConstructor.tagger("span")
A = helpers.XmlConstructor.tagger("a")
UL = helpers.XmlConstructor.tagger("ul")
LI = helpers.XmlConstructor.tagger("li")
LABEL = helpers.XmlConstructor.tagger("label")
XSECTION = helpers.XSECTION
I18N = helpers.I18N
CONCATENATE = helpers.CONCATENATE
XTABLE = widgets.Table
XML = helpers.XML
XTRD = widgets.TableData
XTRH = widgets.TableHead
XFOOTER = widgets.TableFooterPagination


__pragma__('kwargs')


class Index(gatehandler.Handler):
    def __init__(self, request):
        gatehandler.Handler.__init__(self, request)
        self.admin_authorization = localStorage.getItem("admin_authorization")

    def initialize(self):
        html = CONCATENATE(
            DIV(
                DIV(
                    DIV(
                        DIV("TEST REQUERIMENTS", _class="phanterpwa-breadcrumb"),
                        _class="phanterpwa-breadcrumb-wrapper"
                    ),
                    _class="p-container"),
                _class='title_page_container card'
            ),
            DIV(
                DIV(
                    DIV(
                        DIV(
                            widgets.Preloaders("preload_android"),
                            _style="width:100%; text-align: center; padding-top: 100px; padding-bottom: 100px;"
                        ),
                        _id="requeriments_container"),
                    _class="card p-row e-padding_10"
                ),
                _class="phanterpwa-container p-container"
            )
        )
        html.html_to("#main-container")
        BackButton = left_bar.LeftBarButton(
            "back_developer",
            "Voltar",
            I(_class="fas fa-arrow-circle-left"),
            **{
                "_phanterpwa-way": "developer",
                "position": "top",
                "ways": [lambda: window.PhanterPWA.get_current_way().startswith("check_requeriments")]
            }
        )
        window.PhanterPWA.Components['left_bar'].add_button(BackButton)
        window.PhanterPWA.GET(**{
            'url_args': ['api', 'automation', 'requeriment_list'],
            'onComplete': self._after_get_requeriment_list
        })

    def _binds(self):
        jQuery("#phanterpwa-component-left_bar-menu_button-change_projects_folder").off(
            "click.change_projects_folder"
        ).on(
            "click.change_projects_folder",
            self.change_projects_folder,
        )

    def reload(self):
        self._binds()

    def _bind_open_api(self, app_name, url):

        jQuery("#phanterpwa-component-left_bar-menu_button-view_api").off(
            "click.open_href_play"
        ).on(
            "click.open_href_play",
            lambda: window.open(url, "_blank")
        )

    def _xml_requeriment_list(self, json):
        table = XTABLE(
            "requeriments-table",
            XTRH(
                "requeriments-table-head",
                *["Requeriment", "status"]
            )
        )
        for x in json.requeriment_list:
            if ">" in x or "<" in x or "=" in x:
                x = x.split(">")[0]
                x = x.split("=")[0]
                x = x.split("<")[0]
            table.append(
                XTRD(
                    "requeriments-table-data-{0}".format(x),
                    x.capitalize(),
                    DIV(
                        SPAN("Cheking...", _style="color: orange;"),
                        **{
                            "_data-test": x,
                            "_class": "requeriments_phanterpwa_status"
                        }
                    ),
                )
            )

        table.html_to("#requeriments_container")
        window.PhanterPWA.WS.onMessage = lambda ev: self._onwsmessage(ev)
        window.PhanterPWA.WS.send({
            "authorization": self.admin_authorization,
            "command": "requeriments_phanterpwa"
        })

    def _onwsmessage(self, ev):
        json = JSON.parse(ev.data)
        if json.command == "requeriments_phanterpwa":
            if json.status == 206:
                jQuery(".requeriments_phanterpwa_status[data-test='{0}']".format(json.check)).html(
                    "<span style='color: green'>PASS</span>" if json.result else "<span style='color: red'>FAIL</span>"
                )
            elif json.status == 200:
                window.PhanterPWA.flash("Test finished!")

    def _after_get_requeriment_list(self, data, ajax_status):
        if ajax_status == "success":
            json = data.responseJSON
            if data.status == 200:
                window.PhanterPWA.flash(**{'html': json.i18n.message})
                self._xml_requeriment_list(json)
            self._binds()
        else:
            window.PhanterPWA.flash("Problem on server: {0}".format(str(data.status)))


__pragma__('nokwargs')



import setuptools

from phanterpwa import (
    __version__,
    __install_requeriments__
)

with open("README.md", "r") as f:
    long_description = f.read()

def packs(exclude=[]):
    import os
    import glob
    import re
    from pathlib import PurePath
    current_dir = os.path.join(os.path.dirname(__file__), "phanterpwa")
    exclude = exclude
    packs = []
    for x in glob.glob(os.path.join(current_dir, "**"), recursive=True):
        if os.path.isdir(x) and not os.path.basename(x).startswith("_") and "." not in x:
            p = PurePath(x)
            p = p.relative_to(current_dir)
            result = os.path.join(os.path.basename(current_dir), *p.parts)
            if exclude:
                not_passed = []
                for e in exclude:
                    if callable(e):
                        if e(x):
                            not_passed.append(True)
                    elif isinstance(e, str):
                        if any(e == x or os.path.basename(x) == e or re.search(e, x)):
                            not_passed.append(True)
                if not not_passed:
                    packs.append(result.replace("\\", "/"))
            else:
                packs.append(result.replace("\\", "/"))
    return packs


def manin(exclude=[]):
    import os
    import glob
    import re
    from pathlib import PurePath
    exclude = exclude
    current_dir = os.path.join(os.path.dirname(__file__), "phanterpwa")
    files = ""
    for x in glob.glob(os.path.join(current_dir, "**"), recursive=True):
        if os.path.isfile(x) and not os.path.basename(x).endswith(".py") and "__" not in x:
            p = PurePath(x)
            p = p.relative_to(current_dir)
            result = os.path.join(os.path.basename(current_dir), *p.parts)
            if exclude:
                not_passed = []
                for e in exclude:
                    if callable(e):
                        if e(x):
                            not_passed.append(True)
                    elif isinstance(e, str):
                        if any([e == x, os.path.basename(x) == e, re.search(e, x)]):
                            not_passed.append(True)

                if not not_passed:
                    files = "".join([files, "include ", result.replace("\\", "/"), "\n"])


            else:
                files = "".join([files, "include ", result.replace("\\", "/"), "\n"])
    with open("MANIFEST.in", "w", encoding="utf-8") as f:
        f.write(files)
    return files
    
manin(exclude=["sync.ffs_db", "secret.ini", "phanterpwa.pid"])

setuptools.setup(
    name="phanterpwa",
    version=__version__,
    url='https://github.com/PhanterJR/phanterpwa',
    license='MIT',
    author='PhanterJR',
    author_email='phanterjr@conexaodidata.com.br',
    maintainer='PhanterJR',
    maintainer_email='phanterjr@conexaodidata.com.br',
    description='Tools for the phanterpwadeveloper framework',
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=packs(),
    entry_points={
        "console_scripts": [
            "phanterpwa = phanterpwa.__main__:main",
        ],
    },
    install_requires=__install_requeriments__,
    dependency_links=[
        'git+https://github.com/web2py/pydal.git',
    ],
    include_package_data=True,
    exclude_package_data={'': ["sync.ffs_db", "secret.ini", "phanterpwa.pid"]},
    zip_safe=False,
    platforms='any',
    python_requires=">=3.9",
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Text Processing :: Markup :: HTML'
    ],
)



import os
from ...helpers import DIV

PRELOADER = DIV(
    DIV(
        DIV(
            DIV(
                DIV(_class="phanterpwa_explosion phanterpwa_explosion_one"),
                DIV(_class="phanterpwa_explosion phanterpwa_explosion_two"),
                DIV(_class="phanterpwa_explosion phanterpwa_explosion_three"),
                DIV(_class="phanterpwa_explosion phanterpwa_explosion_four"),
                DIV(_class="phanterpwa_explosion phanterpwa_explosion_five"),
                DIV(_class="phanterpwa_explosion phanterpwa_explosion_six"),
                DIV(_class="phanterpwa_explosion phanterpwa_explosion_seven"),
                DIV(_class="phanterpwa_explosion phanterpwa_explosion_eight"),
                DIV(_class="phanterpwa_explosion phanterpwa_explosion_big"),
                _class="phanterpwa_explosion_container"
            ),
            _class='preloader-wrapper enabled'
        ),
        _class="phanterpwa-preloader-wrapper"),
    _class="phanterpwa-components-preloaders-explosion"
)

PRELOADER.sass_file(
    os.path.join(os.path.dirname(__file__), "explosion.sass")
)



import os
import glob
import json
import sys
import tkinter as tk
import psutil
import appdirs
import tempfile
from tkinter import (
    ttk,
    filedialog,
    END,
    VERTICAL,
    LEFT,
    RIGHT,
    Scrollbar,
    Y,
    DISABLED,
    FALSE,
    TRUE
)
from phanterpwa.tools import (
    config,
)
from phanterpwa.interface.cli import package_project_app

from phanterpwa.samples import project_config_sample
from phanterpwa.i18n import Translator
ENV_PYTHON = os.path.normpath(sys.executable)
ENV_PATH = os.path.normpath(os.path.dirname(ENV_PYTHON))
if sys.version_info[0] < 3:
    raise Exception("Must be using Python 3")
PY_VERSION = "{0}.{1}.{2}".format(sys.version_info[0], sys.version_info[1], sys.version_info[2])

CURRENT_DIR = os.path.dirname(__file__)


Trans = Translator(os.path.join(CURRENT_DIR, "langs"), "grafics", debug=True)
T = Trans.T



class ProjectsFolderTk():
    def __init__(self):

        self._projectdata_dir = os.path.join(appdirs.user_data_dir(), "phanterpwa")
        self._projectdata_dir_gui = os.path.join(self._projectdata_dir, "gui")
        if not os.path.isdir(self._projectdata_dir):
            os.makedirs(self._projectdata_dir, exist_ok=True)
        if not os.path.isdir(self._projectdata_dir_gui):
            os.makedirs(self._projectdata_dir_gui, exist_ok=True)
        self._projects_path = None
        self.ProjectsPWA = None
        self.ProjectPWA = None
        self.ConfigAPI = None
        self.ConfigAPP = None
        self.CONFIG = config(self._projectdata_dir_gui)
        app_folder_data = ""
        if "projects_folder" in self.CONFIG:
            if os.path.isdir(self.CONFIG["projects_folder"]):
                self.projects_folder = self.CONFIG["projects_folder"]
                app_folder_data = self.projects_folder
        else:
            self.projects_folder = os.getcwd()
        self.start()

        self.tkInstance.withdraw()
        self.maxheight = 400
        self.ApplicationsTornado = {}
        scrollbar = Scrollbar(self.tkInstance, orient=VERTICAL)
        scrollbar.pack(side=RIGHT, fill=Y, expand=FALSE)
        self.frame_canvas = tk.Canvas(
            self.tkInstance,
            bd=0,
            highlightthickness=0,
            yscrollcommand=scrollbar.set
        )
        self.frame_canvas.config(yscrollcommand=scrollbar.set)
        scrollbar.config(command=self.frame_canvas.yview)
        self.frame_canvas.pack(side=LEFT, fill=Y, expand=TRUE)

        self.tkInstance.title(T("Project Folder - PhanterPWA"))
        self.tkInstance.iconbitmap(os.path.join(CURRENT_DIR, "grafics", "icon.ico"))

        frame = tk.Frame(self.frame_canvas)

        row = 0
        aLabel = ttk.Label(frame, text=T("Where will your projects be saved?"), font='default 14 bold')
        aLabel.grid(column=0, row=row, columnspan=3, padx=(10, 0), pady=(20, 5))

        row += 1
        app_folder_label = ttk.Label(frame, text=T("Path"))
        app_folder_label.grid(column=0, row=row, padx=(20, 0), pady=(5, 20))
        
        self.app_folder_input = ttk.Entry(frame, width=60)
        self.app_folder_input.insert(0, app_folder_data)
        self.app_folder_input.grid(column=1, row=row, padx=(0, 0), pady=(5, 20))
        button = ttk.Button(frame, text=T("Change"), command=lambda: self.setEntryFolder(self.app_folder_input))
        button.grid(column=2, row=row, padx=(0, 20), pady=(5, 20), sticky="w")

        row += 1
        self.buttonOK = ttk.Button(frame, text=T("OK"), command=self.open_window_projects, state=DISABLED)
        self.buttonOK.grid(column=0, row=row, columnspan=3, padx=10, pady=(5, 30))

        self.frame_canvas.create_window((0, 0), window=frame, anchor='nw')
        self.frame_canvas.bind('<Configure>', self.onConfigure)
        self.tkInstance.resizable(0, 0)
        self.tkInstance.maxsize(width=0, height=self.maxheight)
        self.tkInstance.resizable(0, 0)

        self.tkInstance.deiconify()
        self.tkInstance.protocol("WM_DELETE_WINDOW", self.onClosing)
        self.tkInstance.attributes('-topmost', True)
        self.tkInstance.update()
        self.tkInstance.attributes('-topmost', False)
        self.tkInstance.mainloop()

    def setEntryFolder(self, entry_instance):
        if self.projects_folder:
            folder = filedialog.askdirectory(initialdir=self.projects_folder)
        else:
            folder = filedialog.askdirectory()
        if folder:
            self.projects_folder = folder
            self.buttonOK.config(state="normal")
            entry_instance.delete(0, END)
            entry_instance.insert(0, os.path.normpath(folder))

    def onConfigure(self, event):
        height = self.frame_canvas.bbox('all')[3]
        width = self.frame_canvas.bbox('all')[2]
        self.frame_canvas.configure(
            height=height,
            width=width,
            scrollregion=self.frame_canvas.bbox('all')
        )

    def open_window_projects(self):
        print(self.CONFIG)
        if self.app_folder_input.get():
            self.CONFIG = config(self._projectdata_dir_gui, {"projects_folder": os.path.normpath(self.projects_folder)})
            self.tkInstance.destroy()
        #     if not self.ProjectsPWA:
        #         tkInstance = tk.Toplevel(self.tkInstance)
        #         self.ProjectsPWA = ProjectsPWA(tkInstance, self)
        #     else:
        #         self.ProjectsPWA.tkInstance.destroy()
        #         tkInstance = tk.Toplevel(self.tkInstance)
        #         self.ProjectsPWA = ProjectsPWA(tkInstance, self)
        pass

    def start(self):
        self.tkInstance = tk.Tk()
        PID = os.getpid()
        with open("phanterpwa.pid", "w") as f:
            f.write(str(PID))

        if os.path.exists("phanterpwa.pid"):
            with open("phanterpwa.pid", "r") as f:
                pid = f.read().strip()
                if pid:
                    for p in psutil.process_iter():
                        if int(p.pid) != int(pid):
                            cmd_line = None
                            try:
                                cmd_line = p.cmdline()
                            except Exception:
                                pass
                            if cmd_line:
                                basename = os.path.basename
                                if ENV_PYTHON == cmd_line[0]:
                                    if __file__ == cmd_line[-1]:
                                        p.terminate()
                                        print("close server. PID: ", pid)
                                    elif os.path.join(os.path.dirname(__file__), "cli.py") == cmd_line[-1]:
                                        p.terminate()
                                        print("close server. PID: ", pid)
                else:
                    print("Previous server not found")

    def onClosing(self):
        print("Closing")
        self.tkInstance.destroy()


def start():

    ProjectsFolderTk()



if __name__ == '__main__':
    start()



import os
import sys
import appdirs
import urllib.request
import base64
import json
import unittest
import phanterpwa
from phanterpwa.backend.decorators import (
    check_application
)
from phanterpwa import __install_requeriments__ as requeriments
from phanterpwa.configer import ProjectConfig
from phanterpwa.server import ProjectRunner
from phanterpwa.compiler import Compiler
from phanterpwa.tools import (
    humanize_seconds,
    temporary_password,
    config,
    interpolate,
    check_requeriments,
    interpolate,
)
from interface.admin_tk import ProjectsFolderTk
from phanterpwa.i18n import browser_language
from phanterpwa.third_parties.xss import xssescape as E
from tornado import (
    web,
    websocket
)
from pydal import Field
from pydal.validators import (
    IS_EQUAL_TO,
    IS_EMAIL
)
from datetime import (
    datetime,
    timedelta
)
from phanterpwa.tests.tests import test_list
from phanterpwa.tests import (
    test_cli,
    test_components_preloaders,
    test_helpers,
    test_i18n,
    test_reversexml,
    test_tools,
    test_configer
)
from phanterpwa.interface.cli import Projects as ProjectsList
ENV_PYTHON = os.path.normpath(sys.executable)
ENV_PATH = os.path.normpath(os.path.dirname(ENV_PYTHON))
PY_VERSION = "{0}.{1}.{2}".format(sys.version_info[0], sys.version_info[1], sys.version_info[2])

import re


class Projects(web.RequestHandler):
    """
        url: 'api/projects/<?command>'
    """

    def initialize(self, app_name, projectConfig, DALDatabase, i18nTranslator=None, logger_api=None):
        self.app_name = app_name
        self.projectConfig = projectConfig
        self.DALDatabase = DALDatabase
        self.i18nTranslator = i18nTranslator
        if logger_api:
            self.logger_api = logger_api
        if i18nTranslator:
            self.T = i18nTranslator.T
        self.set_header("Access-Control-Allow-Origin", "*")
        self.set_header(
            "Access-Control-Allow-Headers",
            "".join([
                "phanterpwa-language,",
                "phanterpwa-application,",
                "phanterpwa-application-version,",
                "phanterpwa-client-token,",
                "cache-control"
            ])
        )
        self.set_header('Access-Control-Allow-Methods', 'GET, OPTIONS, POST, PUT')
        if self.request.headers.get("phanterpwa-language"):
            self.phanterpwa_language = self.request.headers.get("phanterpwa-language")
        else:
            self.phanterpwa_language = browser_language(self.request.headers.get("Accept-Language"))
        if self.i18nTranslator:
            self.i18nTranslator.direct_translation = self.phanterpwa_language
        self.phanterpwa_user_agent = str(self.request.headers.get('User-Agent'))
        self.phanterpwa_remote_ip = str(self.request.remote_ip)
        self._projectdata_dir = os.path.join(appdirs.user_data_dir(), "phanterpwa")
        self._projectdata_dir_gui = os.path.join(self._projectdata_dir, "gui")
        if not os.path.isdir(self._projectdata_dir):
            os.makedirs(self._projectdata_dir, exist_ok=True)
        if not os.path.isdir(self._projectdata_dir_gui):
            os.makedirs(self._projectdata_dir_gui, exist_ok=True)
        self._projects_path = None

    def check_origin(self, origin):
        return True

    @check_application()
    def get(self, *args, **kargs):
        dict_arguments = {k: self.request.arguments.get(k)[0].decode('utf-8') for k in self.request.arguments}
        self.interfaceConfig = config(self._projectdata_dir_gui)
        if "authorization" in self.interfaceConfig:
            authorization = self.interfaceConfig["authorization"]
        else:
            authorization = os.urandom(12).hex()
            self.interfaceConfig = config(self._projectdata_dir_gui, {"authorization": authorization})
        enviroment = {
            "python_executable": ENV_PYTHON,
            "python_path": ENV_PATH,
            "python_version": PY_VERSION,
            "phanterpwa_version": phanterpwa.__version__,
            "projects_folder": None
        }
        if args and args[0]:
            self._projects_path = self.interfaceConfig["projects_folder"]
            running = ProjectRunner()
            its_running = running.check(os.path.join(self._projects_path, args[0]))
            if os.path.isfile(os.path.join(self._projects_path, args[0], "config.json")):
                config_project = ProjectConfig(os.path.join(self._projects_path, args[0], "config.json"))
                msg = 'Project Found'
                self.set_status(200)
                return self.write({
                    "status": "OK",
                    "message": msg,
                    'i18n': {
                        "message": self.T(msg)
                    },
                    "config": config_project.config,
                    "running": its_running
                })
            else:
                msg = 'Project not Found'
                self.set_status(400)
                return self.write({
                    "status": "Bad Request",
                    "message": msg,
                    'i18n': {
                        "message": self.T(msg)
                    }
                })

        else:
            if "projects_folder" in self.interfaceConfig:
                self._projects_path = self.interfaceConfig["projects_folder"]
                self.Projects = ProjectsList(self._projects_path)
                msg = "Application list"
                enviroment['projects_folder'] = self._projects_path
                running = ProjectRunner()
                self.set_status(200)
                return self.write({
                    "status": "OK",
                    "message": msg,
                    'i18n': {
                        "message": self.T(msg)
                    },
                    "projects_list": [[x[0], x[1], "running" if x[1] in running.projects else
                        "stopped", ProjectConfig(
                            os.path.join(x[1], "config.json"))['PROJECT']['baseport']] for x in self.Projects.projects_list],
                    "authorization": authorization,
                    "enviroment": enviroment,
                })
            else:
                msg = "Get project list"
                self.set_status(202)
                self.write({
                    "status": "Accepted",
                    "message": msg,
                    "authorization": authorization,
                    "enviroment": enviroment,
                    'i18n': {
                        "message": self.T(msg)
                    }
                })

    def options(self, *args):
        self.set_status(200)
        self.write({"status": "OK"})


class Configs(web.RequestHandler):
    """
        url: 'api/config/<?command>'
    """

    def initialize(self, app_name, projectConfig, DALDatabase, i18nTranslator=None, logger_api=None):
        self.app_name = app_name
        self.projectConfig = projectConfig
        self.DALDatabase = DALDatabase
        self.i18nTranslator = i18nTranslator
        if logger_api:
            self.logger_api = logger_api
        if i18nTranslator:
            self.T = i18nTranslator.T
        self.set_header("Access-Control-Allow-Origin", "*")
        self.set_header(
            "Access-Control-Allow-Headers",
            "".join([
                "phanterpwa-language,",
                "phanterpwa-application,",
                "phanterpwa-application-version,",
                "phanterpwa-client-token,",
                "cache-control"
            ])
        )
        self.set_header('Access-Control-Allow-Methods', 'GET, OPTIONS, POST, PUT')
        if self.request.headers.get("phanterpwa-language"):
            self.phanterpwa_language = self.request.headers.get("phanterpwa-language")
        else:
            self.phanterpwa_language = browser_language(self.request.headers.get("Accept-Language"))
        if self.i18nTranslator:
            self.i18nTranslator.direct_translation = self.phanterpwa_language
        self.phanterpwa_user_agent = str(self.request.headers.get('User-Agent'))
        self.phanterpwa_remote_ip = str(self.request.remote_ip)
        self._projectdata_dir = os.path.join(appdirs.user_data_dir(), "phanterpwa")
        self._projectdata_dir_gui = os.path.join(self._projectdata_dir, "gui")
        if not os.path.isdir(self._projectdata_dir):
            os.makedirs(self._projectdata_dir, exist_ok=True)
        if not os.path.isdir(self._projectdata_dir_gui):
            os.makedirs(self._projectdata_dir_gui, exist_ok=True)
        self._projects_path = None

    def check_origin(self, origin):
        return True

    @check_application()
    def get(self, *args, **kargs):
        dict_arguments = {k: self.request.arguments.get(k)[0].decode('utf-8') for k in self.request.arguments}
        self.interfaceConfig = config(self._projectdata_dir_gui)
        if "authorization" in self.interfaceConfig:
            authorization = self.interfaceConfig["authorization"]
        else:
            authorization = os.urandom(12).hex()
            self.interfaceConfig = config(self._projectdata_dir_gui, {"authorization": authorization})
        enviroment = {
            "python_executable": ENV_PYTHON,
            "python_path": ENV_PATH,
            "python_version": PY_VERSION,
            "phanterpwa_version": phanterpwa.__version__,
            "projects_folder": None
        }
        if args and args[0]:
            self._projects_path = self.interfaceConfig["projects_folder"]
            running = ProjectRunner()
            its_running = running.check(os.path.join(self._projects_path, args[0]))
            if os.path.isfile(os.path.join(self._projects_path, args[0], "config.json")):
                config_project = ProjectConfig(os.path.join(self._projects_path, args[0], "config.json"))
                project_ini = config_project.project_ini
                api_ini = config_project.backend_ini
                secret_ini = config_project.frontend_secret_ini
                secret_ini_api = config_project.backend_secret_ini
                project_dict = {}
                api_dict = {}
                secret_dict = {}
                secret_dict_api = {'API': {}}
                for x in project_ini.sections():
                    if x not in project_dict:
                        project_dict[x] = {}

                    for k in project_ini.items(x):
                        v = k[1]
                        if v.isdigit():
                            project_dict[x][k[0]] = int(v)
                        elif v.lower() == "false":
                            project_dict[x][k[0]] = False
                        elif v.lower() == "true":
                            project_dict[x][k[0]] = True
                        else:
                            project_dict[x][k[0]] = v
                for x in api_ini.sections():
                    if x not in api_dict:
                        api_dict[x] = {}
                    for k in api_ini.items(x):
                        v = k[1]
                        if v.isdigit():
                            api_dict[x][k[0]] = int(v)
                        elif v.lower() == "false":
                            api_dict[x][k[0]] = False
                        elif v.lower() == "true":
                            api_dict[x][k[0]] = True
                        else:
                            api_dict[x][k[0]] = v
                for x in secret_ini.sections():
                    if x not in secret_dict:
                        secret_dict[x] = {}
                    for k in secret_ini.items(x):
                        v = k[1]
                        if v.isdigit():
                            secret_dict[x][k[0]] = int(v)
                        elif v.lower() == "false":
                            secret_dict[x][k[0]] = False
                        elif v.lower() == "true":
                            secret_dict[x][k[0]] = True
                        else:
                            secret_dict[x][k[0]] = v

                for k in secret_ini_api.items('API'):
                    v = k[1]
                    if v.isdigit():
                        secret_dict_api['API'][k[0]] = int(v)
                    elif v.lower() == "false":
                        secret_dict_api['API'][k[0]] = False
                    elif v.lower() == "true":
                        secret_dict_api['API'][k[0]] = True
                    else:
                        secret_dict_api['API'][k[0]] = v

                for y in secret_dict:
                    for a in secret_dict[y]:
                        if y not in project_dict:
                            project_dict[y] = {}
                        project_dict[y][a] = secret_dict[y][a]
                for y in secret_dict_api:
                    for a in secret_dict_api[y]:
                        if y not in api_dict:
                            api_dict[y] = {}
                        api_dict[y][a] = secret_dict_api[y][a]
                msg = 'Project Found'
                self.set_status(200)
                return self.write({
                    "status": "OK",
                    "message": msg,
                    'i18n': {
                        "message": self.T(msg)
                    },
                    "project_config": project_dict,
                    "api_config": api_dict,
                    "secret_config": secret_dict
                })
            else:
                msg = 'Project not Found'
                self.set_status(400)
                return self.write({
                    "status": "Bad Request",
                    "message": msg,
                    'i18n': {
                        "message": self.T(msg)
                    }
                })

        else:
            if "projects_folder" in self.interfaceConfig:
                self._projects_path = self.interfaceConfig["projects_folder"]
                self.Projects = ProjectsList(self._projects_path)
                msg = "Application list"
                enviroment['projects_folder'] = self._projects_path
                running = ProjectRunner()
                self.set_status(200)
                return self.write({
                    "status": "OK",
                    "message": msg,
                    'i18n': {
                        "message": self.T(msg)
                    },
                    "projects_list": [[x[0], x[1], "running" if x[1] in running.projects else
                        "stopped"] for x in self.Projects.projects_list],
                    "authorization": authorization,
                    "enviroment": enviroment,
                })
            else:
                msg = "Get project list"
                self.set_status(202)
                self.write({
                    "status": "Accepted",
                    "message": msg,
                    "authorization": authorization,
                    "enviroment": enviroment,
                    'i18n': {
                        "message": self.T(msg)
                    }
                })

    def options(self, *args):
        self.set_status(200)
        self.write({"status": "OK"})


class Automation(web.RequestHandler):
    """
        url: 'api/projects/<?command>'
    """

    def initialize(self, app_name, projectConfig, DALDatabase, i18nTranslator=None, logger_api=None):
        self.app_name = app_name
        self.projectConfig = projectConfig
        self.DALDatabase = DALDatabase
        self.i18nTranslator = i18nTranslator
        if logger_api:
            self.logger_api = logger_api
        if i18nTranslator:
            self.T = i18nTranslator.T
        self.set_header("Access-Control-Allow-Origin", "*")
        self.set_header(
            "Access-Control-Allow-Headers",
            "".join([
                "phanterpwa-language,",
                "phanterpwa-application,",
                "phanterpwa-application-version,",
                "phanterpwa-client-token,",
                "cache-control"
            ])
        )
        self.set_header('Access-Control-Allow-Methods', 'GET, OPTIONS, POST, PUT')
        if self.request.headers.get("phanterpwa-language"):
            self.phanterpwa_language = self.request.headers.get("phanterpwa-language")
        else:
            self.phanterpwa_language = browser_language(self.request.headers.get("Accept-Language"))
        if self.i18nTranslator:
            self.i18nTranslator.direct_translation = self.phanterpwa_language
        self.phanterpwa_user_agent = str(self.request.headers.get('User-Agent'))
        self.phanterpwa_remote_ip = str(self.request.remote_ip)
        self._projectdata_dir = os.path.join(appdirs.user_data_dir(), "phanterpwa")
        self._projectdata_dir_gui = os.path.join(self._projectdata_dir, "gui")
        if not os.path.isdir(self._projectdata_dir):
            os.makedirs(self._projectdata_dir, exist_ok=True)
        if not os.path.isdir(self._projectdata_dir_gui):
            os.makedirs(self._projectdata_dir_gui, exist_ok=True)
        self._projects_path = None

    def check_origin(self, origin):
        return True

    @check_application()
    def get(self, *args, **kargs):
        dict_arguments = {k: self.request.arguments.get(k)[0].decode('utf-8') for k in self.request.arguments}
        self.interfaceConfig = config(self._projectdata_dir_gui)
        if "authorization" in self.interfaceConfig:
            authorization = self.interfaceConfig["authorization"]
        else:
            authorization = os.urandom(12).hex()
            self.interfaceConfig = config(self._projectdata_dir_gui, {"authorization": authorization})
        if args[0] == "test_list":
            msg = 'Test List'
            self.set_status(200)
            return self.write({
                "status": "OK",
                "message": msg,
                "test_list": test_list,
                'i18n': {
                    "message": self.T(msg)
                }
            })
        elif args[0] == "requeriment_list":
            msg = 'Test Requeriments'
            self.set_status(200)
            return self.write({
                "status": "OK",
                "message": msg,
                "requeriment_list": requeriments,
                'i18n': {
                    "message": self.T(msg)
                }
            })

    def options(self, *args):
        self.set_status(200)
        self.write({"status": "OK"})


class EchoWebSocket(websocket.WebSocketHandler):
    connections = set()

    def initialize(self, app_name, projectConfig, DALDatabase, i18nTranslator=None, logger_api=None):
        self.app_name = app_name
        self.projectConfig = projectConfig
        self.DALDatabase = DALDatabase
        self.i18nTranslator = i18nTranslator
        if logger_api:
            self.logger_api = logger_api
        if i18nTranslator:
            self.T = i18nTranslator.T
        self._projectdata_dir = os.path.join(appdirs.user_data_dir(), "phanterpwa")
        self._projectdata_dir_gui = os.path.join(self._projectdata_dir, "gui")

    def check_origin(self, origin):
        return True

    def open(self):
        print("WebSocket opened")
        self.write_message(u"______  _                    _               ______  _    _   ___  ")
        self.write_message(u"| ___ \| |                  | |              | ___ \| |  | | / _ \ ")
        self.write_message(u"| |_/ /| |__    __ _  _ __  | |_   ___  _ __ | |_/ /| |  | |/ /_\ \\")
        self.write_message(u"|  __/ | '_ \  / _` || '_ \ | __| / _ \| '__||  __/ | |/\| ||  _  |")
        self.write_message(u"| |    | | | || (_| || | | || |_ |  __/| |   | |    \  /\  /| | | |")
        self.write_message(u"\_|    |_| |_| \__,_||_| |_| \__| \___||_|   \_|     \/  \/ \_| |_/")
        self.interfaceConfig = config(self._projectdata_dir_gui)
        if "authorization" in self.interfaceConfig:
            self.admin_authorization = self.interfaceConfig["authorization"]
        else:
            self.admin_authorization = os.urandom(12).hex()
            self.interfaceConfig = config(self._projectdata_dir_gui, {"authorization": self.admin_authorization})
        self.connections.add(self)

    def on_message(self, message):
        if message.startswith("{"):
            msg = None
            try:
                json_message = json.loads(message)
            except Exception as e:
                return self.write_message(json.dumps({
                        "status": 400,
                        "error": "SyntaxError",
                        "message": "The message must be json type."
                }))
            else:
                cmd = json_message["command"]
                if "authorization" in json_message and json_message["authorization"] == self.admin_authorization and\
                        "command" in json_message:
                    if cmd == "change_project_folder":
                        current_path = self.interfaceConfig["projects_folder"]
                        ProjectsFolderTk()
                        self.interfaceConfig = config(self._projectdata_dir_gui)
                        if current_path == self.interfaceConfig["projects_folder"]:
                            return self.write_message(json.dumps({
                                "status": 202,
                                "message": "The path not changed.",
                                "command": cmd
                            }))
                        else:
                            return self.write_message(json.dumps({
                                "status": 200,
                                "message": "Changed.",
                                "command": cmd
                            }))
                    elif cmd == "requeriments_phanterpwa":
                        for x in check_requeriments():
                            self.write_message({
                                "status": 206,
                                "check": x[0],
                                "result": x[1],
                                "message": "Pip requeriment \"{0}\" checked".format(x[0]),
                                "command": cmd
                            })
                        self.write_message({
                            "status": 200,
                            "message": "Pip requeriment finished.",
                            "command": cmd
                        })
                    elif cmd == "test_phanterpwa":
                        suite = unittest.TestLoader().loadTestsFromModule(test_cli)
                        result = unittest.TextTestRunner(verbosity=2).run(suite)
                        test_exit_code = bool(int(not result.wasSuccessful()))
                        self.write_message({
                            "status": 206,
                            "test": "test_cli",
                            "result": test_exit_code,
                            "message": "Test cli",
                            "command": cmd
                        })
                        suite = unittest.TestLoader().loadTestsFromModule(test_components_preloaders)
                        result = unittest.TextTestRunner(verbosity=2).run(suite)
                        test_exit_code = bool(int(not result.wasSuccessful()))
                        self.write_message({
                            "status": 206,
                            "test": "test_components_preloaders",
                            "result": test_exit_code,
                            "message": "Test Proloaders",
                            "command": cmd
                        })
                        suite = unittest.TestLoader().loadTestsFromModule(test_helpers)
                        result = unittest.TextTestRunner(verbosity=2).run(suite)
                        test_exit_code = bool(int(not result.wasSuccessful()))
                        self.write_message({
                            "status": 206,
                            "test": "test_helpers",
                            "result": test_exit_code,
                            "message": "Test helpers",
                            "command": cmd
                        })
                        suite = unittest.TestLoader().loadTestsFromModule(test_i18n)
                        result = unittest.TextTestRunner(verbosity=2).run(suite)
                        test_exit_code = bool(int(not result.wasSuccessful()))
                        self.write_message({
                            "status": 206,
                            "test": "test_i18n",
                            "result": test_exit_code,
                            "message": "Test i18n",
                            "command": cmd
                        })
                        suite = unittest.TestLoader().loadTestsFromModule(test_reversexml)
                        result = unittest.TextTestRunner(verbosity=2).run(suite)
                        test_exit_code = bool(int(not result.wasSuccessful()))
                        self.write_message({
                            "status": 206,
                            "test": "test_reversexml",
                            "result": test_exit_code,
                            "message": "Test reversexml",
                            "command": cmd
                        })
                        suite = unittest.TestLoader().loadTestsFromModule(test_tools)
                        result = unittest.TextTestRunner(verbosity=2).run(suite)
                        test_exit_code = bool(int(not result.wasSuccessful()))
                        self.write_message({
                            "status": 206,
                            "test": "test_tools",
                            "result": test_exit_code,
                            "message": "Test tools",
                            "command": cmd
                        })
                        suite = unittest.TestLoader().loadTestsFromModule(test_configer)
                        result = unittest.TextTestRunner(verbosity=2).run(suite)
                        test_exit_code = bool(int(not result.wasSuccessful()))
                        self.write_message({
                            "status": 206,
                            "test": "test_configer",
                            "result": test_exit_code,
                            "message": "Test configer",
                            "command": cmd
                        })

                        self.write_message({
                            "status": 200,
                            "message": "Test Finished",
                            "command": cmd
                        })

                    elif cmd == "compile":
                        _project_path = json_message["project_path"]
                        _application = json_message["application"]
                        c = Compiler(_project_path)
                        has_error = False
                        for x in c.compile_by_step(_application):
                            if not x[1]:
                                has_error = True
                            self.write_message({
                                "status": 206,
                                "project_path": _project_path,
                                "message": x,
                                "command": cmd
                            })
                        return self.write_message({
                            "status": 200 if not has_error else 202,
                            "project_path": _project_path,
                            "message": "Compile complete!" if not has_error else "Has error(s) on compilation",
                            "command": cmd
                        })
                    elif cmd == "run" or cmd == "stop" or cmd == "check" or cmd == "stop_all" or cmd == "check_all":
                        _project_path = json_message["project_path"]
                        R = ProjectRunner()
                        if cmd == "run":
                            R.run(_project_path, thread=True, compile=True)
                        elif cmd == "stop":
                            R.stop(_project_path)

                        p = [x for x in R.projects.keys()]
                        if p:
                            r = {
                                'status': 200,
                                'message': 'Projects running',
                                'command': cmd,
                                'project_running': p
                            }

                            return self.write_message(json.dumps(r))
                        else:
                            return self.write_message({
                                'status': 202,
                                'message': 'All stopped!',
                                'command': cmd
                            })
                else:
                    return self.write_message(json.dumps({
                            "status": 401,
                            "error": "Unauthorized",
                            "message": "Authorization not found.",
                            "command": cmd
                    }))


    def on_close(self):
        self.connections.remove(self)
        print("WebSocket closed")



from pydal import Field
from pydal.validators import (
    IS_NOT_EMPTY,
    IS_EMAIL,
    IS_IN_DB,
    IS_NOT_IN_DB
)
from datetime import datetime


class CASTables():
    def __init__(self, projectConfig, DALDatabase):
        self.DALDatabase = DALDatabase
        self.DALDatabase.define_table('auth_cas',
            Field('email', 'string', notnull=True, unique=True),
            Field('password_hash', 'string', notnull=True, requires=IS_NOT_EMPTY())
        )
        self.DALDatabase.auth_cas.email.requires = [
            IS_EMAIL(),
            IS_NOT_IN_DB(self.DALDatabase, self.DALDatabase.auth_cas.email, error_message="Email already in database.")
        ]
        self.DALDatabase.define_table('cas_activity',
            Field('auth_cas', 'reference auth_cas', requires=IS_IN_DB(self.DALDatabase, self.DALDatabase.auth_cas)),
            Field('request', 'text'),
            Field('activity', 'string'),
            Field('date_activity', 'datetime', default=datetime.now())
        )
        self.DALDatabase.define_table('apps_authorization',
            Field('app_name', 'string'),
            Field('auth_cas', 'reference auth_cas', requires=IS_IN_DB(self.DALDatabase, self.DALDatabase.auth_cas)),
            Field('origin', 'string'),
            Field('authorization', 'text', requires=IS_NOT_EMPTY())
        )
        self.DALDatabase.apps_authorization.app_name.requires = [
            IS_NOT_EMPTY(),
            IS_NOT_IN_DB(self.DALDatabase, self.DALDatabase.apps_authorization.app_name, error_message="Email already in database.")
        ]
